/*
 * $Author: djh $ $Date: 1996/06/18 10:49:40 $
 * $Header: /mac/src/cap60/applications/aufs/RCS/aufsicon.c,v 2.9 1996/06/18 10:49:40 djh Rel djh $
 * $Revision: 2.9 $
 */

/*
 * aufscicon.c - aufs color icon.
 *
 * Set up to display a color CAP volume ICON, and where
 * possible represent the underlying hardware platform.
 *
 * AppleTalk package for UNIX (4.2 BSD).
 *
 * Copyright (c) 1986, 1987 by The Trustees of Columbia University in 
 * the City of New York.
 *
 */

#include <stdio.h>
#include <sys/param.h>
#ifndef _TYPES
 /* assume included by param.h */
# include <sys/types.h>
#endif
#ifdef SOLARIS
# include <unistd.h>
#endif /* SOLARIS */
#ifdef linux
# include <unistd.h>
#endif /* linux */
#include <sys/file.h>
#include <sys/stat.h>
#include <netat/appletalk.h>
#include <netat/compat.h>
#include <netat/afp.h>
#ifdef NEEDFCNTLDOTH
#include <fcntl.h>
#endif /* NEEDFCNTLDOTH */
#include "afps.h"

/*
 * The Icons in this file are intended for non-commercial CAP use,
 * they are provided to visually link the CAP server and host type.
 * Components of the images are copyright by the respective hardware
 * manufacturers.
 *
 * BSD Daemon Copyright 1988 Marshall Kirk McKusick. All Rights Reserved.
 * Penguin With Scarf Copyright 1996 David Hornsby. All Rights Reserved.
 * Used with permisson.
 *
 * To enable the automatic host ICON selection, define USE_HOST_ICON in
 * the m4.features file.
 *
 * The color_cap_icon[] array mirrors the resource fork of a ResEdit file
 * that contains 'icl4', 'icl8', 'ICN#', 'ics#', 'ics4' & 'ics8' resources
 * (resource IDs -16455). If you edit an Icon, or add resource names etc.
 * ensure that the array length is accurate.
 *
 */

#define ICON_FNDR_WAS	286
#define ICON_FNDR_LEN	300
#define ICON_RSRC_LEN	2670
#define ICON_NAME	"Icon:0d"
#define INFO_MESSAGE1	"BSD Daemon Copyright 1988 Marshall Kirk McKusick - \
All Rights Reserved.\r\rhttp://www.cs.mu.OZ.AU/appletalk/cap.html\r"
#define INFO_MESSAGE2	"http://www.cs.mu.OZ.AU/appletalk/cap.html\r"

/* Automatic host/ICON selection */

#ifdef USE_HOST_ICON

#ifdef __NetBSD__
#define BSD_ICON 1
#endif /* __NetBSD__ */
#ifdef __386BSD__
#define BSD_ICON 1
#endif /* __386BSD__ */
#ifdef __FreeBSD__
#define BSD_ICON 1
#endif /* __FreeBSD__ */
#ifdef __bsdi__
#define BSD_ICON 1
#endif /* __bsdi__ */

#ifdef BSD_ICON
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* BSD2icon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x00,0x00,0x80,0x00,0xc0,0xab,0x00,0x08,0x66,0x08,0x30,0x3c,
  0xf8,0xc2,0x60,0x00,0x00,0x88,0x20,0x3c,0x00,0x00,0x80,0x00,0xc0,0xab,
  0x00,0x08,0x66,0x5a,0x20,0x3c,0x09,0x42,0x53,0x44,0x5f,0x50,0x69,0x63,
  0x6f,0x6e,0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,
  0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x3c,0xdf,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x4e,0xba,0x0c,0x44,0x26,0x40,0x4f,0xef,0x00,0x14,0x20,0x3c,0x00,0x00,
  0x80,0x00,0xc0,0xab,0x00,0x08,0x66,0x06,0x30,0x3c,0xf8,0xc2,0x60,0x0e,
  0x20,0x6e,0x00,0x18,0x20,0x8b,0x20,0x6e,0x00,0x14,0x20,0x8a,0x70,0x00,
  0x4c,0xee,0x1c,0xe0,0xff,0xe2,0x4e,0x5e,0x4e,0x75,0x4e,0x56,0xff,0xee,
  0x48,0xe7,0x0f,0x18,0x28,0x2e,0x00,0x1c,0x2a,0x2e,0x00,0x18,0x20,0x6e,
  0x00,0x08,0x26,0x68,0x00,0x40,0x4a,0xae,0x00,0x14,0x67,0x14,0x00,0x84,
  0x00,0x00,0x80,0x00,0x2e,0x05,0x08,0x07,0x00,0x00,0x67,0x0a,0x20,0x07,
  0x52,0x87,0x60,0x04,0x7e,0x00,0x2a,0x07,0x2f,0x0b,0x2f,0x2e,0x00,0x0c,
  0x4e,0xba,0xf6,0x8e,0x2c,0x00,0x50,0x4f,0x66,0x7e,0x20,0x53,0x2c,0x10,
  0x20,0x4b,0x22,0x06,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0xd8,0xff,0x00,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0xff,0xd8,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xff,0xff,0xd8,0xd8,
  0xff,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x00,0xff,0xff,0xff,
  0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,
  0xff,0xff,0x00,0x00,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,
  0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0xff,0xff,0xff,0x00,0x00,0xff,
  0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0xff,0xff,
  0xff,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0x00,0xff,0xff,0xff,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0xff,0xd8,0xff,
  0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,
  0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0xfc,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xd8,0xff,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0x00,
  0xfc,0x00,0x00,0xfc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xff,0x00,0x00,0xfc,0x00,0xfc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xff,
  0xff,0xff,0xd8,0xd8,0xff,0x00,0x00,0x00,0xfc,0xfc,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0xd8,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0xfc,0xfc,0x00,
  0x00,0xfc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xfc,0xfc,
  0xfc,0x00,0x00,0xfc,0xfc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0xff,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0x00,0xff,
  0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0xfc,0xff,0xd8,0xd8,0xd8,0xff,
  0xff,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xd8,0xff,0x00,0x00,0x00,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,
  0xff,0xff,0xff,0xd8,0xff,0x00,0x00,0xff,0x00,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0xff,0xd8,0xff,0xd8,0xff,0xff,0xff,0xff,0xd8,0xff,0xff,0xd8,
  0xd8,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0xff,0x00,0x00,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0xff,0xc0,0xc0,
  0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xfc,0xc0,0xc0,0xc0,
  0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xff,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xfc,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0xff,0xff,0xff,0xfd,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,
  0xff,0xff,0xff,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xf9,
  0xfb,0xfb,0xfb,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0x33,0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,
  0xf9,0xf9,0xf7,0xf7,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0xf6,0xf6,0xf9,0xf9,0xfb,0xfb,0xfb,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0x00,0x00,
  0x00,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x02,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0x0f,0xf0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xf0,0x00,0xff,0xff,0xf0,0x00,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x3f,0x0f,0x33,0x33,
  0x3f,0x0f,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0xf3,
  0x3f,0xff,0x33,0xf3,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0x0f,0xff,0x33,0xf3,0x3f,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x0f,0x00,0xff,0x00,0xf3,0x3f,0x33,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xf0,0x00,0xf0,0x00,0x0f,0x3f,0x33,0x3f,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xf0,0xff,0xf0,0x0f,0x3f,0x33,0x3f,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xf0,0xff,0xf0,0x0f,0x3f,0x33,
  0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xf0,0xff,0xf0,0x0f,
  0x3f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0x00,0xf0,
  0x00,0x0f,0x3f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,
  0x00,0xf0,0x00,0xf3,0x3f,0x33,0x3f,0x00,0x0e,0x00,0x00,0x00,0x00,0x00,
  0x00,0x0f,0xff,0xff,0xff,0x3f,0xf3,0x33,0x3f,0x00,0xe0,0x0e,0x00,0x00,
  0x00,0x00,0x00,0x00,0xf3,0x33,0xf3,0x33,0x33,0x33,0x3f,0x00,0xe0,0xe0,
  0x00,0x00,0x00,0x00,0x00,0x00,0xf3,0x33,0xf3,0x3f,0xff,0x33,0xf0,0x00,
  0xee,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0x33,0x33,0xf3,0x33,
  0xf0,0x0e,0xe0,0x0e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf3,0x33,0x3f,
  0x33,0x3f,0xee,0xe0,0x0e,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,
  0xff,0xff,0xff,0xf3,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xf3,0x33,0x33,0xf3,0x33,0xf0,0x00,0xff,0xf0,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xf3,0x3f,0x33,0x33,0xf0,0x0f,0x00,0x0f,0x00,0x00,
  0xf0,0xff,0x00,0x00,0x0f,0x33,0x3f,0xef,0x33,0x3f,0xf0,0x0f,0x0f,0x0f,
  0x00,0x00,0xff,0xf3,0xf0,0x00,0xff,0x33,0x3f,0x33,0xff,0xf3,0xf0,0x0f,
  0x0f,0xf0,0x00,0x00,0xf3,0xf3,0xff,0xff,0x3f,0xf3,0x3f,0x3f,0x33,0x33,
  0xf0,0xf0,0x0f,0xff,0x00,0x00,0xf3,0xf3,0x33,0x33,0x33,0xff,0xf3,0x33,
  0x33,0x33,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x0f,0x77,0x77,0x77,0x77,0x77,
  0x77,0xe7,0x77,0x77,0x77,0x77,0x77,0x77,0x77,0xf0,0x00,0xff,0xff,0xff,
  0xff,0xff,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xe0,0x00,0x00,0x0a,0xff,0xfa,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xff,0xfa,0x00,0x00,
  0x00,0x00,0xdd,0xee,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xab,0xba,0xbb,
  0xaa,0xae,0xee,0xdd,0xcc,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,
  0xa0,0xab,0xbb,0xbb,0xbb,0xcc,0xdd,0xee,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,
  0xaa,0xaa,0x00,0x0a,0xaa,0xae,0xee,0xdd,0x00,0x00,0x01,0x00,0x00,0x0c,
  0x01,0x80,0x00,0x18,0xf8,0xc0,0x00,0x15,0x05,0x40,0x00,0x12,0x72,0x40,
  0x00,0x0d,0xc9,0x40,0x00,0x13,0x24,0xc0,0x00,0x22,0x14,0x40,0x00,0x3b,
  0x94,0x40,0x00,0x3b,0x94,0x40,0x00,0x3b,0x94,0x40,0x00,0x22,0x14,0x40,
  0x00,0x22,0x24,0x44,0x00,0x1f,0xd8,0x49,0x00,0x08,0x80,0x4a,0x00,0x08,
  0x9c,0x8c,0x00,0x07,0x08,0x99,0x00,0x02,0x11,0xe6,0x00,0x01,0xfe,0x80,
  0x00,0x02,0x08,0x8e,0x00,0x03,0x90,0x91,0x0b,0x04,0x71,0x95,0x0e,0x8c,
  0x4e,0x96,0x0a,0xf6,0x50,0xa7,0x0a,0x03,0x80,0xc0,0xff,0xff,0xff,0xff,
  0x40,0x02,0x00,0x02,0x3f,0xff,0xff,0xfc,0x00,0x08,0x1f,0x00,0x00,0x00,
  0x1f,0x00,0xff,0xff,0xe4,0xff,0x00,0x00,0x0a,0x00,0xff,0xff,0xf1,0xff,
  0x00,0x0c,0x01,0x80,0x00,0x18,0xf8,0xc0,0x00,0x1d,0xfd,0xc0,0x00,0x1f,
  0xff,0xc0,0x00,0x0f,0xff,0xc0,0x00,0x1f,0xff,0xc0,0x00,0x3f,0xff,0xc0,
  0x00,0x3f,0xff,0xc0,0x00,0x3f,0xff,0xc0,0x00,0x3f,0xff,0xc0,0x00,0x3f,
  0xff,0xc0,0x00,0x3f,0xff,0xc4,0x00,0x1f,0xff,0xc9,0x00,0x0f,0xff,0xca,
  0x00,0x0f,0xff,0x8c,0x00,0x07,0xff,0x99,0x00,0x03,0xff,0xe6,0x00,0x01,
  0xff,0x80,0x00,0x03,0xff,0x8e,0x00,0x03,0xff,0x91,0x0b,0x07,0xff,0x95,
  0x0f,0x8f,0xff,0x96,0x0f,0xff,0xff,0xa7,0x0f,0xff,0xff,0xc0,0xff,0xff,
  0xff,0xff,0x7f,0xff,0xff,0xfe,0x3f,0xff,0xff,0xfc,0x00,0x08,0x1f,0x00,
  0x00,0x00,0x1f,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,
  0xf1,0xff,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x00,
  0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0x00,0x00,0xff,0xff,0x00,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0xd8,0xd8,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0xd8,0xd8,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xff,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0xd8,0xff,0xff,0xff,0xd8,
  0xd8,0xd8,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xff,0xd8,0xd8,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xd8,0x00,
  0xd8,0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0xff,0x00,0x00,
  0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,0xd8,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,
  0x00,0x00,0xff,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,
  0xc0,0xc0,0xc0,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x00,0x00,0x00,0x80,
  0x00,0x00,0x0f,0xf0,0x00,0xff,0x00,0x00,0x00,0x00,0x0f,0x00,0xff,0x03,
  0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x0f,0x0f,
  0x0f,0x33,0xf0,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x33,0xf0,0x00,0x00,0x00,
  0x03,0x33,0x33,0x33,0xf0,0x0f,0x00,0x00,0x03,0xff,0xf3,0x33,0xf0,0xf0,
  0x00,0x00,0x00,0x33,0x33,0x3f,0x00,0xff,0x00,0x00,0x00,0x0f,0xf3,0x3f,
  0xff,0x00,0x00,0x30,0x30,0x00,0xf3,0x33,0xf0,0x0f,0x00,0x3f,0x33,0x33,
  0x33,0xf3,0xff,0xff,0x00,0x03,0x33,0x3f,0x33,0x33,0x3f,0x00,0xf7,0x77,
  0x77,0x77,0x77,0x77,0x77,0x7f,0x0f,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,
  0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
  0xaa,0xaa,0x00,0x00,0x00,0x40,0x06,0x30,0x04,0xd0,0x07,0xf0,0x05,0x78,
  0x05,0x78,0x07,0xf9,0x07,0xfa,0x03,0xf3,0x01,0xfc,0x28,0xf9,0x29,0xff,
  0x1f,0xfc,0xff,0xff,0x7f,0xfe,0x00,0x30,0xff,0xff,0x06,0x30,0x04,0xf0,
  0x07,0xf0,0x07,0xf8,0x07,0xf8,0x07,0xf9,0x07,0xfa,0x03,0xf3,0x01,0xfc,
  0x28,0xf9,0x3f,0xff,0x1f,0xfc,0xff,0xff,0x7f,0xfe,0x00,0x30,0xff,0xff,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xee,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x38,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x34,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x38,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x23,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xf3,0x30,0xbf,0xb9,0xff,0xff,0x00,0x00,0x04,0x04,
  0x03,0x71,0xea,0xd8,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf5,0xd4,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xef,0x78,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x08,0x10,0x03,0x70,0xf8,0xc0,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x08,0x94,0x03,0x71,0xf4,0xf0
};
#endif /* BSD_ICON */

#ifndef HOST_ICON
#ifdef  sun
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* SUNicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x08,0xc8,
  0x09,0x99,0x08,0xc8,0x09,0xca,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x07,0x53,0x55,0x4e,0x69,0x63,0x6f,0x6e,
  0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,
  0x00,0x00,0x00,0x81,0x00,0x00,0x00,0x00,0x00,0x23,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0x00,0x00,0x00,0x81,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x3b,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x00,0x00,0x00,0x53,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x02,0x03,0x04,0x01,0x02,
  0x03,0x04,0x01,0x02,0x03,0x04,0x01,0x02,0x03,0x04,0x01,0x02,0x03,0x04,
  0x01,0x02,0x03,0x04,0x01,0x02,0x03,0x04,0x01,0x02,0x03,0x04,0x01,0x02,
  0x03,0x04,0x01,0x02,0x03,0x04,0x01,0x02,0x03,0x04,0x01,0x02,0x03,0x04,
  0x00,0x07,0x07,0x43,0x6f,0x75,0x72,0x69,0x65,0x72,0x02,0x05,0x06,0x02,
  0x05,0x07,0x03,0x05,0x06,0x07,0x01,0x2d,0x04,0x42,0x6f,0x6c,0x64,0x07,
  0x4f,0x62,0x6c,0x69,0x71,0x75,0x65,0x00,0x00,0x00,0x01,0x00,0x00,0x02,
  0x1a,0xf6,0x00,0x02,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,
  0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x00,0x7f,0x7f,0x7f,0x00,
  0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,
  0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,
  0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,
  0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x00,0x7f,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,
  0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,
  0x7f,0x00,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x00,
  0x7f,0x7f,0x7f,0x00,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,
  0x7f,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,
  0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,
  0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,
  0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x7f,0x7f,
  0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0xab,
  0xab,0xab,0xab,0x00,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x00,
  0x00,0x00,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,
  0x7f,0x00,0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,0x00,0x00,0x00,0x00,
  0x7f,0x7f,0x7f,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,
  0x7f,0x00,0x7f,0x7f,0x7f,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
  0xab,0xab,0xab,0x00,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,
  0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x7f,0x00,0x7f,0x7f,0x7f,0x7f,
  0x7f,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,
  0xab,0x2a,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,
  0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,
  0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,
  0xab,0x00,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x00,
  0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0xff,0x2a,0x2a,0xff,
  0x2a,0xff,0x2a,0x2a,0xab,0x00,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,
  0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x7f,0x7f,0x7f,0x00,
  0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,
  0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x00,
  0x00,0x00,0x00,0x00,0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,0x00,
  0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x7f,
  0x00,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x7f,0x7f,
  0x7f,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x7f,0x7f,0x7f,0x00,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,
  0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xfb,
  0xfb,0x33,0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,
  0xf9,0xf9,0xf7,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0xf7,0xf7,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x02,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x55,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x55,0x50,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,
  0x05,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x50,0x55,0x50,0x55,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x05,0x55,0x05,0x55,0x05,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x50,0x55,0x50,0x55,0x50,0x55,0x50,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x05,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x50,0x55,0x50,0x55,0x50,0x55,0x50,0x50,
  0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x55,0x05,0x55,0x05,0x55,0x05,
  0x55,0x05,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x50,0x50,0x55,0x50,
  0x55,0x50,0x50,0x55,0x50,0x50,0x00,0x00,0x00,0x00,0x05,0x55,0x05,0x55,
  0x05,0x55,0x05,0x50,0x05,0x55,0x05,0x55,0x00,0x00,0x00,0x00,0x55,0x50,
  0x55,0x50,0x00,0x55,0x55,0x50,0x55,0x50,0x55,0x50,0x00,0x00,0x00,0x05,
  0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x00,
  0x00,0x00,0x00,0x55,0x50,0x55,0x55,0x50,0x00,0x55,0x50,0x55,0x50,0x55,
  0x55,0x50,0x0e,0xee,0xe0,0x55,0x05,0x55,0x05,0x50,0x00,0x55,0x05,0x55,
  0x05,0x55,0x05,0x50,0xe7,0x77,0x7e,0x00,0x00,0x00,0x55,0x50,0x00,0x55,
  0x55,0x50,0x55,0x50,0x55,0x50,0xee,0xee,0xee,0xee,0xee,0xe0,0x55,0x05,
  0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x00,0xe7,0x77,0x77,0x77,0x77,0xe0,
  0x50,0x55,0x55,0x50,0x00,0x55,0x50,0x55,0x50,0x00,0xe7,0x77,0xf7,0x77,
  0x77,0xe0,0x00,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x00,0x00,0xe7,0x7f,
  0x77,0x77,0x77,0xe0,0x50,0x55,0x50,0x55,0x50,0x50,0x55,0x50,0x00,0x00,
  0xe7,0xf7,0x7f,0x7f,0x77,0xe0,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x00,
  0x00,0x00,0xe7,0x77,0x77,0x77,0x77,0xe0,0x55,0x50,0x55,0x50,0x55,0x50,
  0x50,0x00,0x00,0x00,0xee,0xee,0xee,0xee,0xee,0xe0,0x05,0x55,0x05,0x55,
  0x05,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0xad,0xa0,0x00,0x00,0x50,0x55,
  0x50,0x55,0x50,0x50,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x05,
  0x55,0x05,0x55,0x05,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0x3f,
  0x00,0x00,0x55,0x50,0x55,0x50,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,
  0xff,0xff,0x00,0x00,0x05,0x55,0x05,0x50,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x0a,0xda,0xda,0x00,0x00,0x00,0x55,0x55,0x50,0x00,0x00,0x00,0x00,
  0x00,0x00,0xde,0xad,0xda,0xdd,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,
  0xaa,0xae,0xee,0xdd,0xcd,0xdd,0xa0,0xad,0xdd,0xdd,0xdd,0xdd,0xdd,0xdd,
  0xdd,0xdd,0xdd,0xdd,0xdd,0xcc,0xde,0xaa,0x00,0x0a,0xaa,0xaa,0xaa,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0x00,0x00,0x01,0x00,0x00,0x00,
  0x00,0x00,0x00,0x01,0xc0,0x00,0x00,0x03,0xe0,0x00,0x00,0x03,0x70,0x00,
  0x00,0x0b,0xb8,0x00,0x00,0x1d,0xdc,0x00,0x00,0x2e,0xee,0x00,0x00,0x77,
  0x77,0x00,0x00,0xbb,0xba,0x80,0x01,0xdd,0xdd,0xc0,0x03,0xae,0xeb,0xa0,
  0x07,0x77,0x67,0x70,0x0e,0xe3,0xee,0xe0,0x1d,0xdd,0xdd,0xdc,0x03,0xbe,
  0x3b,0xbe,0x7b,0x76,0x37,0x76,0x84,0x0e,0x3e,0xee,0xff,0xed,0xdd,0xdc,
  0x80,0x2b,0xe3,0xb8,0x88,0x23,0x77,0x70,0x90,0x2b,0xba,0xe0,0xa5,0x2d,
  0xdd,0xc0,0x80,0x2e,0xee,0x80,0xff,0xe7,0x77,0x00,0x0a,0x0b,0xba,0x00,
  0x1f,0x1d,0xdc,0x00,0x11,0x0e,0xe8,0x00,0x1f,0x07,0x60,0x00,0x15,0x03,
  0xe0,0x00,0xe4,0xff,0xff,0xff,0x0a,0x00,0x00,0x00,0xf1,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x03,0xe0,0x00,0x00,0x07,0xf0,0x00,0x00,0x0b,
  0xf8,0x00,0x00,0x1f,0xfc,0x00,0x00,0x3f,0xfe,0x00,0x00,0x7f,0xff,0x00,
  0x00,0xff,0xff,0x80,0x01,0xff,0xff,0xc0,0x03,0xff,0xff,0xe0,0x07,0xff,
  0xff,0xf0,0x0f,0xff,0xef,0xf8,0x1f,0xf7,0xff,0xf4,0x3f,0xff,0xff,0xfe,
  0x03,0xff,0xff,0xfe,0x7b,0xff,0xff,0xfe,0xfc,0x0f,0xff,0xfe,0xff,0xef,
  0xff,0xfe,0xff,0xef,0xf7,0xfc,0xff,0xeb,0xff,0xf8,0xff,0xef,0xff,0xf0,
  0xff,0xef,0xff,0xe0,0xff,0xef,0xff,0xc0,0xff,0xef,0xff,0x80,0x0e,0x0f,
  0xff,0x00,0x1f,0x3f,0xfe,0x00,0x1f,0x1f,0xfc,0x00,0x1f,0x0f,0xe8,0x00,
  0x1f,0x07,0xf0,0x00,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,0xf1,0xff,
  0xff,0xff,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,
  0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,
  0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,
  0x7f,0x7f,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,
  0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0xab,0xab,0xab,0x7f,0x7f,0x7f,0x7f,0x7f,
  0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0xab,0xab,0xab,0xab,0xab,0xab,
  0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0xab,0x2a,0xff,0x2a,
  0x2a,0xab,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0xab,0xff,
  0xff,0xff,0x2a,0xab,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,
  0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x7f,0x7f,0x7f,0x7f,
  0x7f,0x00,0x00,0x00,0x00,0x00,0xfb,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0xfb,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xf9,0x00,0x00,0x00,0x80,
  0x00,0x00,0x00,0x05,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x55,0x00,
  0x00,0x00,0x00,0x00,0x05,0x55,0x55,0x50,0x00,0x00,0x00,0x00,0x55,0x55,
  0x55,0x55,0x00,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x50,0x00,0x00,0x55,
  0x55,0x55,0x55,0x55,0x55,0x00,0x05,0x55,0x55,0x55,0x55,0x55,0x55,0x50,
  0xee,0xe5,0x55,0x55,0x05,0x55,0x55,0x55,0xee,0xee,0xee,0x55,0x55,0x55,
  0x55,0x55,0xe7,0xf7,0x7e,0x55,0x55,0x55,0x55,0x50,0xef,0xff,0x7e,0x55,
  0x55,0x55,0x55,0x00,0xee,0xee,0xee,0x55,0x55,0x55,0x50,0x00,0x0f,0xff,
  0x05,0x55,0x55,0x55,0x00,0x00,0x0f,0xff,0x00,0x55,0x55,0x50,0x00,0x00,
  0xed,0xdd,0xaa,0xaa,0xaa,0xaa,0xae,0xed,0xed,0xad,0xdd,0xdd,0xdd,0xdd,
  0xde,0xed,0x00,0x00,0x00,0x40,0x01,0x80,0x03,0xc0,0x07,0xe0,0x0f,0xf0,
  0x1f,0xf8,0x3f,0xfc,0x7f,0xfe,0xff,0x7f,0xff,0xff,0xa7,0xfe,0xf7,0xfc,
  0xff,0xf8,0x77,0xf0,0x73,0xe0,0xff,0xff,0xff,0xff,0x01,0xc0,0x03,0xe0,
  0x07,0xf0,0x0f,0xf8,0x1f,0xfc,0x3f,0xfe,0x7f,0xff,0xff,0xff,0xff,0xff,
  0xff,0xfe,0xff,0xfc,0xff,0xf8,0x7f,0xf0,0x7f,0xe0,0xff,0xff,0xff,0xff,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xee,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x38,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x34,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x38,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x23,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xf1,0xbc,0xbf,0xb9,0xff,0xff,0x00,0x00,0x04,0x04,
  0x03,0x71,0xf2,0xd4,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf1,0x7c,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xf1,0x74,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x08,0x10,0x03,0x71,0xf5,0x4c,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x08,0x94,0x03,0x71,0xf4,0xac
};
#endif /* sun */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#ifdef  NeXT
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* NeXTicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x6e,0x00,0x08,0x32,0x2e,0x00,0x12,0x60,0x06,0x12,0xc2,0x30,
  0x01,0x53,0x41,0x4a,0x41,0x66,0xf6,0x4e,0x5e,0x4e,0x75,0x4e,0x56,0xff,
  0xfc,0x48,0xe7,0x03,0x08,0x42,0x08,0x4e,0x45,0x58,0x54,0x69,0x63,0x6f,
  0x6e,0x58,0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,
  0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x36,0xfb,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x6f,0x00,0x04,0x70,0x00,0xa0,0xac,0x20,0x5f,0x58,0x8f,0x60,0x0c,0x30,
  0x6f,0x00,0x04,0x70,0x01,0xa0,0xac,0x20,0x5f,0x54,0x8f,0x3e,0x80,0x4e,
  0xd0,0x48,0xe7,0x18,0x00,0x20,0x6f,0x00,0x0c,0x36,0x2f,0x00,0x10,0x38,
  0x2f,0x00,0x12,0x70,0x05,0xa0,0xac,0x4c,0xdf,0x00,0x18,0x20,0x5f,0x50,
  0x8f,0x60,0xde,0x48,0xe7,0x18,0x00,0x20,0x6f,0x00,0x10,0x30,0x10,0x20,
  0x6f,0x00,0x0c,0x31,0x40,0x00,0x04,0x36,0x2f,0x00,0x14,0x38,0x2f,0x00,
  0x16,0x70,0x02,0xa0,0xac,0x32,0x04,0x4c,0xdf,0x00,0x18,0x4a,0x40,0x66,
  0x06,0x20,0x6f,0x00,0x08,0x30,0x81,0x20,0x5f,0xde,0xfc,0x00,0x0c,0x60,
  0xa8,0x2f,0x03,0x20,0x6f,0x00,0x08,0x36,0x2f,0x00,0x0c,0x70,0x06,0xa0,
  0xac,0x26,0x1f,0x20,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0xf1,0xf0,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x1f,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0xf1,0xff,
  0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0x1f,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,
  0xff,0xf1,0xff,0xff,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0xff,0xff,0x1f,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x0f,0xff,0xff,0xf1,0xff,0xff,0xf1,0x11,0xff,0xff,0xf0,0x00,0x00,
  0x00,0x00,0x00,0xff,0xff,0xff,0x1f,0xff,0xff,0xf1,0x1f,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0xf1,0xff,0xff,0xff,0xf1,0xff,0x1f,
  0xff,0xff,0xf0,0x00,0x00,0x00,0xff,0xff,0xff,0x1f,0xff,0xff,0x1f,0xff,
  0x11,0xff,0x1f,0xff,0xff,0x00,0x00,0x0f,0xff,0xff,0xf1,0xff,0xff,0xff,
  0xf1,0xff,0xff,0xf1,0xff,0xff,0xff,0xf0,0x00,0x0f,0xff,0xff,0x1f,0xff,
  0xff,0x11,0x11,0x1f,0xff,0x1f,0x1f,0xff,0xff,0xff,0x0e,0xee,0xef,0xff,
  0xf1,0xff,0xff,0xf1,0xff,0xff,0xff,0xff,0xf1,0xff,0xff,0xf0,0xe7,0x77,
  0x7e,0xff,0xff,0x1f,0xff,0xff,0x1f,0xff,0x1f,0xff,0xff,0xff,0xff,0x00,
  0xee,0xee,0xee,0xee,0xee,0xe1,0xff,0xff,0xff,0xff,0x1f,0xff,0xff,0xff,
  0xf0,0x00,0xe7,0x77,0x77,0x77,0x77,0xe0,0x1f,0xff,0xff,0x11,0x11,0x1f,
  0xff,0xff,0x00,0x00,0xe7,0x77,0xf7,0x77,0x77,0xe0,0xf1,0xff,0xff,0xff,
  0x1f,0xff,0xff,0xf0,0x00,0x00,0xe7,0x7f,0x77,0x77,0x77,0xe0,0xff,0x1f,
  0xff,0xff,0x1f,0xff,0xff,0x00,0x00,0x00,0xe7,0xf7,0x7f,0x7f,0x77,0xe0,
  0xff,0xf1,0xff,0xff,0xff,0xff,0xf0,0x00,0x00,0x00,0xe7,0x77,0x77,0x77,
  0x77,0xe0,0xff,0xff,0x1f,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xee,0xee,
  0xee,0xee,0xee,0xe0,0xff,0xff,0xf1,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,
  0x00,0x00,0xad,0xa0,0x00,0x00,0xff,0xff,0xff,0x1f,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x0f,0xff,0xff,0xf1,0xf0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0x3f,0x00,0x00,0x00,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x0f,
  0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xba,0xba,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xde,0xeb,0xba,0xbb,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcb,0xbb,
  0xa0,0xab,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xde,0xea,0x00,0x0a,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
  0xff,0x05,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,
  0xff,0xff,0xff,0xff,0xff,0x05,0x05,0x05,0xff,0xff,0xff,0xff,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
  0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0x05,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x05,
  0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,
  0x05,0xff,0xff,0xff,0x05,0x05,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
  0x05,0xff,0xff,0xff,0xff,0xff,0x05,0x05,0x05,0x05,0x05,0xff,0xff,0xff,
  0x05,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0xab,0xab,0xab,
  0xab,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0x00,
  0xab,0x54,0x54,0x54,0x54,0xab,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,
  0xff,0xff,0x05,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
  0xab,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x05,0xff,0xff,0xff,0xff,0xff,0x05,0x05,
  0x05,0x05,0x05,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xab,0x2a,
  0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0xff,0x05,0xff,0xff,
  0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0xab,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,
  0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0xff,0x2a,0x2a,0xff,0x2a,0xff,
  0x2a,0x2a,0xab,0x00,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0xff,0xff,0xff,0xff,0x05,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0xff,0xff,
  0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x05,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,0xfd,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xfb,0xfb,0x33,
  0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf7,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf7,0xf7,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0xf0,0x00,0x00,0x07,0xe8,0x00,
  0x00,0x0f,0xdc,0x00,0x00,0x1f,0xbe,0x00,0x00,0x3f,0x7f,0x00,0x00,0x7e,
  0xff,0x80,0x00,0xfd,0xff,0xc0,0x01,0xfb,0xe3,0xe0,0x03,0xf7,0xe7,0xf0,
  0x07,0xef,0xed,0xf8,0x0f,0xdf,0x73,0x7c,0x1f,0xbf,0xbe,0xfe,0x1f,0x7c,
  0x1d,0x7f,0x7f,0xbe,0xff,0xbe,0x87,0xdf,0x77,0xfc,0xff,0xef,0xf7,0xf8,
  0x80,0x27,0xc1,0xf0,0x88,0x2b,0xf7,0xe0,0x90,0x2d,0xf7,0xc0,0xa5,0x2e,
  0xff,0x80,0x80,0x2f,0x7f,0x00,0xff,0xef,0xbe,0x00,0x0a,0x0f,0xdc,0x00,
  0x1f,0x07,0xe8,0x00,0x11,0x03,0xf0,0x00,0x1f,0x01,0xe0,0x00,0x15,0x00,
  0x00,0x00,0xe4,0xff,0xff,0xff,0x0a,0x00,0x00,0x00,0xf1,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0xf0,0x00,0x00,0x07,
  0xf8,0x00,0x00,0x0f,0xfc,0x00,0x00,0x1f,0xfe,0x00,0x00,0x3f,0xff,0x00,
  0x00,0x7f,0xff,0x80,0x00,0xff,0xff,0xc0,0x01,0xff,0xff,0xe0,0x03,0xff,
  0xff,0xf0,0x07,0xff,0xff,0xf8,0x0f,0xff,0xff,0xfc,0x1f,0xff,0xff,0xfe,
  0x1f,0xff,0xff,0xff,0x7f,0xff,0xff,0xfe,0xff,0xff,0xff,0xfc,0xff,0xff,
  0xff,0xf8,0xff,0xef,0xff,0xf0,0xff,0xff,0xff,0xe0,0xff,0xef,0xff,0xc0,
  0xff,0xff,0xff,0x80,0xff,0xef,0xff,0x00,0xff,0xff,0xfe,0x00,0x0e,0x0f,
  0xfc,0x00,0x1f,0x07,0xf8,0x00,0x1f,0x03,0xf0,0x00,0x1f,0x01,0xe0,0x00,
  0x1f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,0xf1,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0x00,0x00,0x03,0xe0,0x07,0x70,0x0e,0xf8,
  0x1d,0x9c,0x3b,0xfe,0x76,0x67,0xfb,0xff,0xfd,0x9e,0xa6,0xfc,0xf7,0x78,
  0xff,0xb0,0x73,0xe0,0x71,0xc0,0xff,0xfe,0xff,0xfe,0x00,0x00,0x03,0xe0,
  0x07,0xf0,0x0f,0xf8,0x1f,0xfc,0x3f,0xfe,0x7f,0xff,0xff,0xff,0xff,0xfe,
  0xff,0xfc,0xff,0xf8,0xff,0xf0,0x7f,0xe0,0x7f,0xc0,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x0f,0xff,0x1f,0xff,0x00,0x00,
  0x00,0x00,0xff,0xf1,0xff,0xff,0xf0,0x00,0x00,0x0f,0xff,0x1f,0xf1,0x1f,
  0xff,0x00,0x00,0xff,0xf1,0xff,0xff,0xff,0xff,0xf0,0x0f,0xff,0x1f,0xf1,
  0x1f,0xf1,0x1f,0xff,0xef,0xff,0xf1,0xff,0xff,0xff,0xff,0xff,0xee,0xef,
  0xff,0x1f,0xf1,0x1f,0xff,0xf0,0xec,0xfc,0xce,0xf1,0xff,0xff,0xff,0x00,
  0xef,0xff,0xce,0xff,0x1f,0xff,0xf0,0x00,0xee,0xee,0xee,0xff,0xf1,0xff,
  0x00,0x00,0x0f,0xff,0x00,0xff,0xff,0xf0,0x00,0x00,0x0f,0xff,0x00,0x0f,
  0xff,0x00,0x00,0x00,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xed,0xea,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xed,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x05,0xff,0xff,0xff,0xff,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x05,0xff,0xff,0x05,
  0x05,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x05,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x05,0xff,
  0xff,0x05,0x05,0xff,0xff,0x05,0x05,0xff,0xff,0xff,0xab,0xff,0xff,0xff,
  0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xab,0xab,
  0xab,0xff,0xff,0xff,0x05,0xff,0xff,0x05,0x05,0xff,0xff,0xff,0xff,0x00,
  0xab,0x2a,0xff,0x2a,0x2a,0xab,0xff,0x05,0xff,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0xab,0xff,0xff,0xff,0x2a,0xab,0xff,0xff,0x05,0xff,0xff,0xff,
  0xff,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xff,0xff,0xff,0x05,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,
  0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0xfb,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xf9,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xf6,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xf1,0xe4,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x03,0x71,0xee,0xb0,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf3,0xdc,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xf5,0x84,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x03,0x71,0xf0,0x2c,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x03,0x71,0xf0,0xe4
};
#endif /* NeXT */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#ifdef  AIX
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* IBMicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x20,0x03,0x9d,
  0x1f,0x00,0x00,0x00,0x00,0xf0,0x20,0xff,0xff,0x00,0x03,0x9d,0x27,0x00,
  0x00,0x00,0x00,0xf0,0x37,0xff,0x07,0x49,0x42,0x4d,0x69,0x63,0x6f,0x6e,
  0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,
  0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x1d,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x37,0x63,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x00,0x00,0x00,0xf0,0x3e,0xff,0xff,0x00,0x03,0x9f,0x1c,0x00,0x00,0x00,
  0x00,0xf0,0x3c,0xff,0xff,0x00,0x03,0x9f,0x84,0x00,0x00,0x00,0x00,0xf0,
  0x30,0xff,0xff,0x00,0x03,0xa0,0x12,0x00,0x00,0x00,0x00,0xf0,0x32,0xff,
  0xff,0x00,0x03,0xa5,0x06,0x00,0x00,0x00,0x00,0xf0,0x20,0xff,0xff,0x00,
  0x03,0xa6,0x0e,0x00,0x00,0x00,0x00,0xf0,0x3f,0xff,0xff,0x00,0x03,0xaf,
  0x41,0x00,0x00,0x00,0x00,0xf0,0x37,0xff,0xff,0x00,0x03,0xf6,0x52,0x00,
  0x00,0x00,0x00,0xf0,0x20,0xff,0xff,0x00,0x03,0xaf,0x5f,0x00,0x00,0x00,
  0x00,0xf0,0x20,0xff,0xff,0x00,0x03,0xaf,0x81,0x00,0x00,0x00,0x00,0xf0,
  0x37,0xff,0xff,0x00,0x03,0xb0,0x85,0x00,0x00,0x00,0x00,0xf0,0x38,0x01,
  0x8d,0x00,0x03,0xb1,0x00,0x00,0x02,0x00,0x06,0x66,0x60,0x06,0x66,0x66,
  0x66,0x00,0x00,0x66,0x66,0x60,0x00,0x66,0x66,0x60,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x66,
  0x60,0x06,0x66,0x66,0x66,0x66,0x00,0x66,0x66,0x60,0x00,0x66,0x66,0x60,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x66,0x00,0x00,0x66,0x00,0x00,0x66,0x60,0x06,0x66,0x60,
  0x00,0x66,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x66,0x00,0x00,0x66,0x66,0x66,0x66,
  0x00,0x06,0x66,0x60,0x00,0x66,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x66,0x00,0x00,
  0x66,0x66,0x66,0x66,0x00,0x06,0x60,0x66,0x06,0x60,0x66,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x66,0x00,0x00,0x66,0x00,0x00,0x66,0x60,0x06,0x60,0x66,0x06,0x60,
  0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x06,0x66,0x60,0x06,0x66,0x66,0x66,0x66,0x00,0x66,
  0x60,0x06,0x66,0x00,0x66,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x66,0x60,0x06,0x66,0x66,
  0x66,0x00,0x00,0x66,0x60,0x00,0x60,0x00,0x66,0x60,0x0e,0xee,0xe0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x77,
  0x7e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xee,0xee,0xee,0xee,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xe7,0x77,0x77,0x77,0x77,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xe7,0x77,0xf7,0x77,0x77,0xe0,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x7f,0x77,0x77,0x77,0xe0,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0xf7,0x7f,0x7f,0x77,0xe0,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x77,0x77,0x77,
  0x77,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xee,0xee,
  0xee,0xee,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xba,0xba,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xde,0xeb,0xba,0xbb,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcb,0xbb,
  0xa0,0xab,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xde,0xea,0x00,0x0a,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0x00,0xec,
  0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,
  0xec,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xec,0xec,0xec,0xec,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0xec,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0xec,0xec,
  0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x00,0x00,
  0x00,0x00,0xec,0xec,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0x00,0x00,0xec,
  0xec,0xec,0xec,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xec,0xec,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,
  0xec,0xec,0xec,0xec,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0x00,0x00,
  0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,
  0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,
  0x00,0xec,0xec,0x00,0xec,0xec,0x00,0xec,0xec,0x00,0xec,0xec,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x00,0x00,0x00,0x00,0xec,0xec,
  0x00,0x00,0x00,0x00,0xec,0xec,0xec,0x00,0x00,0xec,0xec,0x00,0xec,0xec,
  0x00,0xec,0xec,0x00,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,
  0xec,0xec,0xec,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,
  0x00,0x00,0xec,0xec,0xec,0x00,0x00,0xec,0xec,0xec,0x00,0x00,0xec,0xec,
  0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0x00,0xec,
  0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0x00,
  0x00,0x00,0xec,0x00,0x00,0x00,0xec,0xec,0xec,0x00,0x00,0xab,0xab,0xab,
  0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
  0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,
  0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0xff,0x2a,0x2a,0xff,0x2a,0xff,
  0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,0xfd,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xfb,0xfb,0x33,
  0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf7,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf7,0xf7,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0x79,0xfc,
  0x3e,0x3e,0x00,0x00,0x00,0x00,0x79,0xff,0x3e,0x3e,0x00,0x00,0x00,0x00,
  0x30,0xc3,0x9e,0x3c,0x00,0x00,0x00,0x00,0x30,0xff,0x1e,0x3c,0x00,0x00,
  0x00,0x00,0x30,0xff,0x1b,0x6c,0x00,0x00,0x00,0x00,0x30,0xc3,0x9b,0x6c,
  0x00,0x00,0x00,0x00,0x79,0xff,0x39,0xce,0x00,0x00,0x00,0x00,0x79,0xfc,
  0x38,0x8e,0x78,0x00,0x00,0x00,0x84,0x00,0x00,0x00,0xff,0xe0,0x00,0x00,
  0x80,0x20,0x00,0x00,0x88,0x20,0x00,0x00,0x90,0x20,0x00,0x00,0xa5,0x20,
  0x00,0x00,0x80,0x20,0x00,0x00,0xff,0xe0,0x00,0x00,0x0a,0x00,0x00,0x00,
  0x1f,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x15,0x00,
  0x00,0x00,0xe4,0xff,0xff,0xff,0x0a,0x00,0x00,0x00,0xf1,0xff,0xff,0xff,
  0x79,0xfc,0x3e,0x3e,0x79,0xfc,0x3e,0x3e,0x79,0xff,0x3e,0x3e,0x30,0xc3,
  0x1e,0x3c,0x30,0xc3,0x9e,0x3c,0x30,0xc3,0x1e,0x3c,0x30,0xff,0x1e,0x3c,
  0x30,0xfe,0x1a,0x2c,0x30,0xff,0x1b,0x6c,0x30,0xc3,0x1b,0x6c,0x30,0xc3,
  0x9b,0x6c,0x30,0xc3,0x19,0xcc,0x79,0xff,0x39,0xce,0x79,0xfc,0x38,0x8e,
  0x79,0xfc,0x38,0x8e,0x78,0x00,0x00,0x00,0xfc,0x00,0x00,0x00,0xff,0xe0,
  0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,
  0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0x0e,0x00,
  0x00,0x00,0x1f,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,
  0x1f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,0xf1,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0xfe,0x77,0xff,0x77,0x49,0xf6,0x4f,0x76,
  0x4f,0x7e,0x49,0xfe,0xff,0x7b,0xfe,0x6b,0xfc,0x00,0xa4,0x00,0xf4,0x00,
  0xfc,0x00,0x70,0x00,0x70,0x00,0xff,0xff,0xff,0xff,0xfe,0x77,0xff,0x77,
  0x49,0xf6,0x4f,0x76,0x4f,0x7e,0x49,0xfe,0xff,0x7b,0xfe,0x6b,0xfc,0x00,
  0xfc,0x00,0xfc,0x00,0xfc,0x00,0x70,0x00,0x70,0x00,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x66,0x66,0x66,0x60,0x06,0x66,0x06,0x66,0x66,0x66,
  0x66,0x66,0x06,0x66,0x06,0x66,0x06,0x00,0x60,0x06,0x66,0x66,0x06,0x60,
  0x06,0x00,0x66,0x66,0x06,0x66,0x06,0x60,0x06,0x00,0x66,0x66,0x06,0x66,
  0x66,0x60,0x06,0x00,0x60,0x06,0x66,0x66,0x66,0x60,0x66,0x66,0x66,0x66,
  0x06,0x66,0x60,0x66,0xee,0xe6,0x66,0x60,0x06,0x60,0x60,0x66,0xee,0xee,
  0xee,0x00,0x00,0x00,0x00,0x00,0xe7,0xf7,0x7e,0x00,0x00,0x00,0x00,0x00,
  0xef,0xff,0x7e,0x00,0x00,0x00,0x00,0x00,0xee,0xee,0xee,0x00,0x00,0x00,
  0x00,0x00,0x0f,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0xee,0xbb,0xaa,0xaa,0xaa,0xaa,0xae,0xed,0xee,0xab,
  0xbb,0xbb,0xbb,0xbb,0xbe,0xed,0x00,0x00,0x01,0x00,0xec,0xec,0xec,0xec,
  0xec,0xec,0xec,0x00,0x00,0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0xec,
  0xec,0xec,0xec,0xec,0xec,0xec,0x00,0xec,0xec,0xec,0x00,0xec,0xec,0xec,
  0x00,0xec,0x00,0x00,0xec,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0x00,0xec,
  0xec,0x00,0x00,0xec,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0xec,0xec,0xec,
  0x00,0xec,0xec,0x00,0x00,0xec,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0xec,
  0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xec,0x00,0x00,0xec,0x00,0x00,0xec,
  0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0xec,0x00,0xec,0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0xec,0xec,0x00,0x00,0xec,0xec,0x00,0xec,0x00,0xec,0xec,0xab,0xab,
  0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0x2a,0xff,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0xff,0xff,0xff,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0xfb,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xf9,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xf6,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xf1,0x94,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x03,0x71,0xee,0xb8,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf1,0x8c,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xe0,0xa8,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x03,0x71,0xf5,0x58,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x03,0x71,0xf5,0x64
};
#endif /* AIX */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#if defined(ultrix) || defined(vax) || defined(__alpha)
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* DIGITALicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0xff,0xff,0xff,0xff,0x00,0x9c,0x00,0xd2,0x11,0x01,0xa0,0x00,
  0x82,0xa0,0x00,0x8c,0x01,0x00,0x0a,0xff,0xff,0xff,0xff,0x00,0x9c,0x00,
  0xd2,0x09,0x00,0x00,0x00,0x00,0x0b,0x44,0x49,0x47,0x49,0x54,0x41,0x4c,
  0x69,0x63,0x6f,0x6e,0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,
  0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x37,0xc3,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x00,0x09,0x00,0x03,0x06,0x47,0x65,0x6e,0x65,0x76,0x61,0x03,0x00,0x03,
  0x0d,0x00,0x0c,0x2b,0x0e,0x40,0x1e,0x54,0x68,0x65,0x20,0x42,0x75,0x69,
  0x6c,0x74,0x2d,0x49,0x6e,0x20,0x44,0x69,0x67,0x69,0x74,0x69,0x7a,0x65,
  0x72,0x20,0x63,0x61,0x6e,0x6e,0x6f,0x74,0x20,0x2b,0x0a,0x10,0x1b,0x64,
  0x69,0x73,0x70,0x6c,0x61,0x79,0x20,0x76,0x69,0x64,0x65,0x6f,0x20,0x77,
  0x68,0x69,0x6c,0x65,0x20,0x69,0x6e,0x20,0x74,0x68,0x65,0x20,0x2b,0x02,
  0x10,0x19,0x63,0x75,0x72,0x72,0x65,0x6e,0x74,0x20,0x6e,0x75,0x6d,0x62,
  0x65,0x72,0x20,0x6f,0x66,0x20,0x63,0x6f,0x6c,0x6f,0x72,0x73,0x2e,0xa0,
  0x00,0x97,0xa0,0x00,0x8d,0xa0,0x00,0x83,0xff,0x00,0x00,0x00,0xe4,0x00,
  0xe4,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x33,0x33,0x30,0x33,0x30,0x33,
  0x33,0x30,0x33,0x30,0x33,0x30,0x33,0x33,0x03,0x33,0x33,0x33,0x30,0x33,
  0x30,0x33,0x33,0x30,0x33,0x30,0x33,0x30,0x33,0x33,0x03,0x33,0x33,0x30,
  0x30,0x33,0x30,0x33,0x33,0x30,0x33,0x30,0x33,0x30,0x33,0x33,0x03,0x03,
  0x33,0x30,0x30,0x30,0x30,0x33,0x33,0x30,0x30,0x30,0x30,0x30,0x33,0x33,
  0x03,0x03,0x33,0x30,0x30,0x33,0x30,0x33,0x33,0x30,0x33,0x30,0x30,0x30,
  0x30,0x03,0x03,0x03,0x30,0x00,0x30,0x30,0x30,0x30,0x00,0x30,0x30,0x30,
  0x00,0x00,0x33,0x30,0x03,0x03,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
  0x30,0x30,0x30,0x30,0x30,0x00,0x03,0x03,0x30,0x30,0x30,0x30,0x30,0x30,
  0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x03,0x03,0x30,0x00,0x30,0x30,
  0x30,0x30,0x00,0x30,0x30,0x30,0x33,0x00,0x30,0x00,0x03,0x03,0x33,0x33,
  0x30,0x33,0x30,0x33,0x30,0x30,0x33,0x30,0x33,0x30,0x33,0x33,0x03,0x33,
  0x33,0x33,0x30,0x33,0x30,0x33,0x30,0x30,0x33,0x30,0x33,0x30,0x33,0x33,
  0x03,0x33,0x33,0x33,0x30,0x33,0x30,0x30,0x03,0x30,0x33,0x30,0x33,0x30,
  0x33,0x33,0x03,0x33,0x33,0x33,0x30,0x33,0x30,0x33,0x33,0x30,0x33,0x30,
  0x33,0x30,0x33,0x33,0x03,0x33,0x33,0x33,0x30,0x33,0x30,0x33,0x33,0x30,
  0x33,0x30,0x33,0x30,0x33,0x33,0x03,0x33,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0e,0xee,0xe0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x77,
  0x7e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xee,0xee,0xee,0xee,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xe7,0x77,0x77,0x77,0x77,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xe7,0x77,0xf7,0x77,0x77,0xe0,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x7f,0x77,0x77,0x77,0xe0,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0xf7,0x7f,0x7f,0x77,0xe0,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x77,0x77,0x77,
  0x77,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xee,0xee,
  0xee,0xee,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xad,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xba,0xba,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xde,0xeb,0xba,0xbb,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcb,0xbb,
  0xa0,0xab,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xde,0xea,0x00,0x0a,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,
  0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,
  0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,
  0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,
  0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,
  0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,
  0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,
  0x00,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,
  0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,
  0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x00,
  0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,
  0x6a,0x00,0x6a,0x00,0x6a,0x00,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x6a,0x00,
  0x00,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x00,0x00,0x6a,0x00,
  0x6a,0x00,0x6a,0x00,0x00,0x00,0x00,0x00,0x6a,0x6a,0x6a,0x00,0x00,0x6a,
  0x00,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,
  0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,
  0x00,0x00,0x00,0x6a,0x00,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,
  0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,
  0x6a,0x00,0x6a,0x00,0x6a,0x00,0x00,0x6a,0x00,0x6a,0x6a,0x00,0x00,0x00,
  0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x00,0x00,0x6a,0x00,0x6a,0x00,
  0x6a,0x00,0x6a,0x6a,0x00,0x00,0x6a,0x00,0x00,0x00,0x00,0x6a,0x00,0x6a,
  0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,
  0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,
  0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,
  0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,
  0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,
  0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x00,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,
  0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,
  0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,
  0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,
  0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,
  0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x6a,0x6a,
  0x6a,0x6a,0x00,0x6a,0x6a,0x6a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0xab,0xab,
  0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
  0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,
  0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0xff,0x2a,0x2a,0xff,0x2a,0xff,
  0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,0xfd,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xfb,0xfb,0x33,
  0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf7,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf7,0xf7,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0xfb,0xbe,
  0xee,0xf7,0xfb,0xbe,0xee,0xf7,0xeb,0xbe,0xee,0xf5,0xea,0xbe,0xaa,0xf5,
  0xeb,0xbe,0xea,0x95,0x8a,0xa2,0xa0,0xe5,0xaa,0xaa,0xaa,0x85,0xaa,0xaa,
  0xaa,0xa5,0x8a,0xa2,0xac,0x85,0xfb,0xba,0xee,0xf7,0xfb,0xba,0xee,0xf7,
  0xfb,0xa6,0xee,0xf7,0xfb,0xbe,0xee,0xf7,0xfb,0xbe,0xee,0xf7,0x00,0x00,
  0x00,0x00,0x78,0x00,0x00,0x00,0x84,0x00,0x00,0x00,0xff,0xe0,0x00,0x00,
  0x80,0x20,0x00,0x00,0x88,0x20,0x00,0x00,0x90,0x20,0x00,0x00,0xa5,0x20,
  0x00,0x00,0x80,0x20,0x00,0x00,0xff,0xe0,0x00,0x00,0x0a,0x00,0x00,0x00,
  0x1f,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x15,0x00,
  0x00,0x00,0xe4,0xff,0xff,0xff,0x0a,0x00,0x00,0x00,0xf1,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x78,0x00,0x00,0x00,0xfc,0x00,0x00,0x00,0xff,0xe0,
  0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,
  0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0x0e,0x00,
  0x00,0x00,0x1f,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,
  0x1f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,0xf1,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0xff,0xff,0xaf,0x5e,0xbf,0xce,0xa9,0x52,
  0x29,0x52,0xfd,0xff,0xfb,0xff,0xe0,0x00,0xfc,0x00,0xa4,0x00,0xf4,0x00,
  0xfc,0x00,0x70,0x00,0x70,0x00,0xff,0xfe,0xff,0xfe,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe0,0x00,0xfc,0x00,
  0xfc,0x00,0xfc,0x00,0xfc,0x00,0x70,0x00,0x70,0x00,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x30,0x30,
  0x33,0x33,0x03,0x03,0x33,0x30,0x30,0x33,0x33,0x33,0x33,0x00,0x33,0x30,
  0x30,0x30,0x30,0x03,0x03,0x03,0x00,0x30,0x00,0x30,0x30,0x03,0x03,0x03,
  0x00,0x30,0x33,0x33,0x33,0x03,0x33,0x33,0x33,0x33,0x33,0x33,0x30,0x33,
  0x33,0x33,0x33,0x33,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0xee,0xee,
  0xee,0x00,0x00,0x00,0x00,0x00,0xec,0xfc,0xce,0x00,0x00,0x00,0x00,0x00,
  0xef,0xff,0xce,0x00,0x00,0x00,0x00,0x00,0xee,0xee,0xee,0x00,0x00,0x00,
  0x00,0x00,0x0f,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xed,0xea,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xed,0x00,0x00,0x01,0x00,0x6a,0x6a,0x6a,0x6a,
  0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,
  0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x00,
  0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x00,0x6a,0x6a,
  0x6a,0x00,0x6a,0x00,0x6a,0x00,0x6a,0x00,0x00,0x6a,0x00,0x6a,0x00,0x6a,
  0x00,0x00,0x6a,0x00,0x00,0x00,0x6a,0x00,0x6a,0x00,0x00,0x6a,0x00,0x6a,
  0x00,0x6a,0x00,0x00,0x6a,0x00,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,0x6a,
  0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x00,
  0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0x6a,0xab,0xab,0xab,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0xab,
  0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0x2a,0xff,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0xff,0xff,0xff,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0xfb,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xf6,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xf0,0x9c,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x03,0x71,0xf3,0x28,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf4,0xfc,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xf0,0x80,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x03,0x71,0xf5,0x78,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x03,0x71,0xf0,0x08
};
#endif /* ultrix || vax || __alpha */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#ifdef  aux
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* AUXicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x41,0x55,0x58,0x69,0x63,0x6f,0x6e,
  0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,
  0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x1f,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x38,0x21,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x08,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x88,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x80,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x88,0x80,0x08,0x80,0x08,0x88,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x88,0x88,0x88,0x08,0x80,0x88,
  0x88,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x88,0x88,0x88,0x88,
  0x88,0x88,0x88,0x88,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x11,0x11,0x11,
  0x11,0x11,0x11,0x11,0x11,0x11,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x11,
  0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x10,0x00,0x00,0x00,0x00,0x00,
  0x01,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x10,0x00,0x00,0x00,0x00,
  0x00,0x00,0x02,0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x22,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
  0x22,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x30,0x00,0x0e,0xee,0xe0,0x00,
  0x00,0x04,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x40,0x00,0xe7,0x77,
  0x7e,0x00,0x00,0x04,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x40,0x00,
  0xee,0xee,0xee,0xee,0xee,0xe0,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,
  0x00,0x00,0xe7,0x77,0x77,0x77,0x77,0xe0,0x00,0x66,0x66,0x66,0x66,0x66,
  0x66,0x60,0x00,0x00,0xe7,0x77,0xf7,0x77,0x77,0xe0,0x00,0x06,0x66,0x60,
  0x66,0x66,0x66,0x00,0x00,0x00,0xe7,0x7f,0x77,0x77,0x77,0xe0,0x00,0x00,
  0x66,0x00,0x06,0x66,0x00,0x00,0x00,0x00,0xe7,0xf7,0x7f,0x7f,0x77,0xe0,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe7,0x77,0x77,0x77,
  0x77,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xee,0xee,
  0xee,0xee,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xad,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xba,0xba,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xde,0xeb,0xba,0xbb,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcb,0xbb,
  0xa0,0xab,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xde,0xea,0x00,0x0a,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe3,
  0xe3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xe3,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xe3,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xe3,0xe3,0xe3,0xe3,0x00,0x00,0xe3,0xe3,0x00,0x00,0xe3,
  0xe3,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0x00,0xe3,
  0xe3,0x00,0xe3,0xe3,0xe3,0xe3,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe3,0xe3,0xe3,0xe3,0xe3,
  0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x17,0x17,0x17,0x17,
  0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,
  0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
  0x17,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
  0x17,0x17,0x17,0x17,0x17,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0xab,0xab,0xab,
  0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
  0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,0x00,0x00,
  0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,0x00,0x00,0x00,0x20,0x20,0x20,
  0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
  0x20,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,
  0xab,0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
  0x20,0x20,0x20,0x20,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,
  0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0xec,
  0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,
  0x00,0x00,0xab,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,
  0x00,0x00,0x00,0x00,0xec,0xec,0x00,0x00,0x00,0xec,0xec,0xec,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0xff,0x2a,0x2a,0xff,0x2a,0xff,
  0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,0xfd,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xfb,0xfb,0x33,
  0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf7,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf7,0xf7,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0x00,0x00,
  0x06,0x00,0x00,0x00,0x0c,0x00,0x00,0x00,0x18,0x00,0x00,0x07,0x99,0xc0,
  0x00,0x0f,0xdb,0xe0,0x00,0x1f,0xff,0xf0,0x00,0x3f,0xff,0xf8,0x00,0x3f,
  0xff,0xf8,0x00,0x7f,0xff,0xe0,0x00,0x7f,0xff,0xc0,0x00,0x7f,0xff,0xc0,
  0x00,0x7f,0xff,0xc0,0x00,0x7f,0xff,0xc0,0x00,0x3f,0xff,0xe0,0x00,0x3f,
  0xff,0xf8,0x78,0x1f,0xff,0xf8,0x84,0x1f,0xff,0xf8,0xff,0xef,0xff,0xf0,
  0x80,0x23,0xff,0xe0,0x88,0x21,0xef,0xc0,0x90,0x20,0xc7,0x00,0xa5,0x20,
  0x00,0x00,0x80,0x20,0x00,0x00,0xff,0xe0,0x00,0x00,0x0a,0x00,0x00,0x00,
  0x1f,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x15,0x00,
  0x00,0x00,0xe4,0xff,0xff,0xff,0x0a,0x00,0x00,0x00,0xf1,0xff,0xff,0xff,
  0x00,0x00,0x06,0x00,0x00,0x00,0x0c,0x00,0x00,0x00,0x18,0x00,0x00,0x07,
  0x99,0xc0,0x00,0x0f,0xdb,0xe0,0x00,0x1f,0xff,0xf0,0x00,0x3f,0xff,0xf8,
  0x00,0x3f,0xff,0xf8,0x00,0x7f,0xff,0xe0,0x00,0x7f,0xff,0xc0,0x00,0x7f,
  0xff,0xc0,0x00,0x7f,0xff,0xc0,0x00,0x7f,0xff,0xc0,0x00,0x3f,0xff,0xe0,
  0x00,0x3f,0xff,0xf8,0x78,0x1f,0xff,0xf8,0xfc,0x0f,0xff,0xf8,0xff,0xe7,
  0xff,0xf0,0xff,0xe3,0xff,0xe0,0xff,0xe1,0xef,0xc0,0xff,0xe0,0xc7,0x00,
  0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0xff,0xe0,0x00,0x00,0x0e,0x00,
  0x00,0x00,0x1f,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,
  0x1f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,0xf1,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0x00,0x30,0x03,0xf8,0x07,0xfc,0x07,0xfe,
  0x0f,0xfc,0x0f,0xf8,0x0f,0xfc,0xe7,0xfe,0xff,0xfe,0xa5,0xfc,0xf4,0xb0,
  0xfc,0x00,0x70,0x00,0x70,0x00,0xff,0xff,0xff,0xff,0x00,0x30,0x03,0xf8,
  0x07,0xfc,0x07,0xfe,0x0f,0xfc,0x0f,0xf8,0x0f,0xfc,0xe7,0xfe,0xff,0xfe,
  0xfd,0xfc,0xfc,0xb0,0xfc,0x00,0x70,0x00,0x70,0x00,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0x00,0x88,0x00,0x00,0x00,0x00,
  0x00,0x88,0x88,0x88,0x80,0x00,0x00,0x00,0x08,0x88,0x88,0x88,0x88,0x00,
  0x00,0x00,0x01,0x11,0x11,0x11,0x11,0x10,0x00,0x00,0x22,0x22,0x22,0x22,
  0x21,0x00,0x00,0x00,0x22,0x22,0x22,0x22,0x20,0x00,0x00,0x00,0x33,0x33,
  0x33,0x33,0x33,0x00,0xee,0xe0,0x04,0x44,0x44,0x44,0x44,0x40,0xee,0xee,
  0xee,0x44,0x44,0x44,0x44,0x40,0xe7,0xf7,0x7e,0x06,0x66,0x66,0x66,0x00,
  0xef,0xff,0x7e,0x00,0x60,0x66,0x00,0x00,0xee,0xee,0xee,0x00,0x00,0x00,
  0x00,0x00,0x0f,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0xee,0xbb,0xaa,0xaa,0xaa,0xaa,0xae,0xed,0xee,0xab,
  0xbb,0xbb,0xbb,0xbb,0xbe,0xed,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xe3,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x17,0x17,0x17,0x17,0x17,0x17,
  0x17,0x17,0x17,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x17,0x17,0x17,
  0x17,0x17,0x17,0x17,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xab,0xab,0xab,0x00,
  0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0xab,0xab,
  0xab,0xab,0xab,0xab,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x00,
  0xab,0x2a,0xff,0x2a,0x2a,0xab,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,
  0x00,0x00,0xab,0xff,0xff,0xff,0x2a,0xab,0x00,0x00,0xec,0x00,0xec,0xec,
  0x00,0x00,0x00,0x00,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0xfb,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xf9,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xf6,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xd6,0x90,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x03,0x71,0xf2,0x24,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf4,0xd8,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xf6,0x48,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x03,0x71,0xf5,0x2c,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x03,0x71,0xef,0xbc
};
#endif /* aux */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#ifdef  sgi
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* SGIicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x08,0x00,0x09,0x37,0x07,0xe7,0x0b,0x68,0x0d,0x2b,0x08,0xc8,
  0x04,0x62,0x05,0x53,0x05,0x53,0x0c,0x49,0x0b,0xce,0x08,0xe5,0x07,0x1a,
  0x05,0x53,0x09,0xb2,0x08,0xc8,0x07,0x53,0x47,0x49,0x69,0x63,0x6f,0x6e,
  0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,0x05,0x00,
  0x00,0x68,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x05,0x00,0x00,0x68,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x38,0x96,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x06,0x39,0x06,0x39,0x06,0x39,0x0d,0x53,0x0d,0x53,0x0c,0xa3,0x0d,0x53,
  0x0c,0x72,0x0c,0x72,0x0c,0x72,0x05,0x53,0x05,0x53,0x05,0x53,0x05,0x53,
  0x05,0x53,0x05,0x53,0x08,0xe5,0x05,0x53,0x05,0x53,0x05,0x53,0x05,0x53,
  0x04,0x00,0x00,0x00,0x00,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x02,0x03,0x04,0x05,0x02,0x03,0x04,0x05,0x02,0x03,0x04,0x05,0x02,0x03,
  0x04,0x05,0x02,0x03,0x04,0x05,0x02,0x03,0x04,0x05,0x02,0x03,0x04,0x05,
  0x02,0x03,0x04,0x05,0x02,0x03,0x04,0x05,0x02,0x03,0x04,0x05,0x02,0x03,
  0x04,0x05,0x02,0x03,0x04,0x05,0x00,0x09,0x08,0x50,0x61,0x6c,0x61,0x74,
  0x69,0x6e,0x6f,0x02,0x06,0x07,0x02,0x06,0x08,0x02,0x06,0x09,0x03,0x06,
  0x08,0x09,0x01,0x2d,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x55,0x50,0x00,0x55,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x05,0x55,0x55,0x05,0x55,0x55,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x55,0x50,0x55,0x05,0x50,0x55,0x50,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x55,0x00,0x55,0x05,0x50,0x05,0x55,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x50,0x00,0x55,0x05,0x50,
  0x00,0x55,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x00,0x00,0x55,
  0x05,0x50,0x00,0x05,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x50,0x55,0x50,
  0x00,0x55,0x05,0x50,0x00,0x55,0x50,0x50,0x00,0x00,0x00,0x00,0x05,0x55,
  0x05,0x55,0x00,0x55,0x05,0x50,0x05,0x55,0x05,0x55,0x00,0x00,0x00,0x00,
  0x05,0x55,0x50,0x55,0x50,0x55,0x05,0x50,0x55,0x50,0x55,0x55,0x00,0x00,
  0x00,0x00,0x05,0x55,0x55,0x05,0x55,0x05,0x05,0x05,0x55,0x05,0x55,0x55,
  0x00,0x00,0x00,0x00,0x05,0x50,0x55,0x50,0x55,0x50,0x00,0x55,0x50,0x55,
  0x50,0x55,0x00,0x00,0x00,0x00,0x05,0x50,0x05,0x55,0x05,0x55,0x05,0x55,
  0x05,0x55,0x00,0x55,0x00,0x00,0x00,0x00,0x05,0x50,0x00,0x55,0x50,0x55,
  0x55,0x50,0x55,0x50,0x00,0x55,0x0e,0xee,0xe0,0x00,0x05,0x50,0x00,0x05,
  0x55,0x05,0x55,0x05,0x55,0x00,0x00,0x55,0xe7,0x77,0x7e,0x00,0x05,0x50,
  0x00,0x50,0x55,0x50,0x50,0x55,0x50,0x50,0x00,0x55,0xee,0xee,0xee,0xee,
  0xee,0xee,0x05,0x55,0x05,0x55,0x05,0x55,0x05,0x55,0x00,0x55,0xe7,0x77,
  0x77,0x77,0x77,0x7e,0x55,0x50,0x50,0x55,0x05,0x50,0x50,0x55,0x50,0x55,
  0xe7,0x77,0x77,0x7f,0x77,0x7e,0x55,0x05,0x55,0x55,0x05,0x55,0x55,0x05,
  0x55,0x55,0xe7,0x77,0x77,0xf7,0x77,0x7e,0x50,0x55,0x50,0x55,0x05,0x50,
  0x55,0x50,0x55,0x55,0xe7,0x77,0x7f,0x77,0x77,0x7e,0x05,0x55,0x00,0x55,
  0x05,0x50,0x05,0x55,0x05,0x55,0xe7,0x77,0xf7,0x77,0x77,0x7e,0x55,0x50,
  0x00,0x55,0x05,0x50,0x00,0x55,0x50,0x50,0xe7,0x7f,0x77,0xf7,0xf7,0x7e,
  0x55,0x00,0x00,0x55,0x05,0x50,0x00,0x05,0x50,0x00,0xe7,0x77,0x77,0x77,
  0x77,0x7e,0x55,0x50,0x00,0x55,0x05,0x50,0x00,0x55,0x50,0x00,0xee,0xee,
  0xee,0xee,0xee,0xee,0x05,0x55,0x00,0x55,0x05,0x50,0x05,0x55,0x00,0x00,
  0x00,0x00,0x0a,0xca,0x00,0x00,0x00,0x55,0x50,0x55,0x05,0x50,0x55,0x50,
  0x00,0x00,0x00,0x00,0xff,0xff,0xf0,0x00,0x00,0x05,0x55,0x55,0x05,0x55,
  0x55,0x00,0x00,0x00,0x00,0x00,0xf3,0x33,0xf0,0x00,0x00,0x00,0x55,0x50,
  0x00,0x55,0x50,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xf0,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0xab,0xa0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xdd,0xee,0xbb,0xab,
  0xba,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcc,0xbb,
  0xba,0x0a,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xdd,0xee,0xa0,0x00,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,
  0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,
  0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,
  0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,
  0x00,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,0x00,
  0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0x00,
  0xb0,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,
  0xb0,0xb0,0xb0,0x00,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0x00,0xb0,0xb0,0x00,0xb0,
  0xb0,0x00,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,
  0xb0,0xb0,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0xb0,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,
  0x00,0xb0,0xb0,0xb0,0x00,0xb0,0x00,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,
  0xb0,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,
  0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,
  0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xb0,0xb0,0x00,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,
  0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0x00,0xb0,0xb0,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,
  0xb0,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,
  0xb0,0xb0,0x00,0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,
  0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,
  0x00,0x00,0x00,0x00,0xb0,0xb0,0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,
  0x00,0xb0,0xb0,0x00,0x00,0x00,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0x00,
  0xb0,0xb0,0xb0,0x00,0xb0,0x00,0x00,0x00,0xb0,0xb0,0xab,0xab,0xab,0xab,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0xb0,0xb0,0xb0,0x00,0xb0,
  0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0x00,0xb0,0xb0,
  0xab,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0xb0,0xb0,
  0xb0,0x00,0xb0,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0xb0,0x00,0xb0,0xb0,
  0xb0,0x00,0xb0,0xb0,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xfe,0x2a,0x2a,
  0x2a,0xab,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,
  0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0xfe,0x2a,0x2a,0x2a,0x2a,0xab,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,
  0x00,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0xb0,0xab,0x2a,
  0x2a,0x2a,0x2a,0xfe,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0xb0,0xb0,0xb0,
  0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0xb0,0xb0,0xb0,0x00,0xb0,
  0xb0,0xb0,0xab,0x2a,0x2a,0x2a,0xfe,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,
  0xb0,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,
  0xb0,0xb0,0xb0,0x00,0xb0,0x00,0xab,0x2a,0x2a,0xfe,0x2a,0x2a,0xfe,0x2a,
  0xfe,0x2a,0x2a,0xab,0xb0,0xb0,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,0xb0,
  0xb0,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0xb0,0xb0,0xb0,0x00,0x00,0x00,
  0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0xb0,
  0xb0,0xb0,0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0xb0,0xb0,0xb0,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0xf8,0xfd,0x00,0x00,
  0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,
  0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,
  0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xb0,0xb0,0xb0,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,
  0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xf9,0xfb,0xfb,
  0x33,0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf6,0xf6,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf6,0xf6,0xf9,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0x00,0x00,
  0xe3,0x80,0x00,0x01,0xf7,0xc0,0x00,0x03,0xb6,0xe0,0x00,0x07,0x36,0x70,
  0x00,0x0e,0x36,0x38,0x00,0x0c,0x36,0x18,0x00,0x2e,0x36,0x3a,0x00,0x77,
  0x36,0x77,0x00,0x7b,0xb6,0xef,0x00,0x7d,0xd5,0xdf,0x00,0x6e,0xe3,0xbb,
  0x00,0x67,0x77,0x73,0x00,0x63,0xbe,0xe3,0x78,0x61,0xdd,0xc3,0x84,0x62,
  0xeb,0xa3,0xff,0xf7,0x77,0x73,0x80,0x1e,0xb6,0xbb,0x81,0x1d,0xf7,0xdf,
  0x82,0x1b,0xb6,0xef,0x84,0x17,0x36,0x77,0x88,0x1e,0x36,0x3a,0x92,0x9c,
  0x36,0x18,0x80,0x1e,0x36,0x38,0xff,0xf7,0x36,0x70,0x05,0x03,0xb6,0xe0,
  0x0f,0x81,0xf7,0xc0,0x0f,0x80,0xe3,0x80,0x0f,0x80,0x00,0x00,0x0a,0x80,
  0x00,0x00,0x32,0x7f,0xff,0xfc,0x05,0x00,0x00,0x00,0x38,0xff,0xff,0xfc,
  0x00,0x00,0xe3,0x80,0x00,0x01,0xf7,0xc0,0x00,0x03,0xb6,0xe0,0x00,0x07,
  0x36,0x70,0x00,0x0e,0x36,0x38,0x00,0x0c,0x36,0x18,0x00,0x2e,0x36,0x3a,
  0x00,0x77,0x36,0x77,0x00,0x7b,0xb6,0xef,0x00,0x7d,0xd5,0xdf,0x00,0x6e,
  0xe3,0xbb,0x00,0x67,0x77,0x73,0x00,0x63,0xbe,0xe3,0x78,0x61,0xdd,0xc3,
  0xfc,0x62,0xeb,0xa3,0xff,0xf7,0x77,0x73,0xff,0xfe,0xb6,0xbb,0xff,0xfd,
  0xf7,0xdf,0xff,0xfb,0xb6,0xef,0xff,0xf7,0x36,0x77,0xff,0xfe,0x36,0x3a,
  0xff,0xfc,0x36,0x18,0xff,0xfe,0x36,0x38,0xff,0xf7,0x36,0x70,0x07,0x03,
  0xb6,0xe0,0x0f,0x81,0xf7,0xc0,0x0f,0x80,0xe3,0x80,0x0f,0x80,0x00,0x00,
  0x0f,0x80,0x00,0x00,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xf8,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0x01,0xf8,0x03,0xfc,0x07,0x76,0x0f,0x77,
  0x0f,0xff,0x0f,0xff,0xed,0xdd,0xfd,0xfd,0x97,0xff,0xb7,0xff,0xff,0x77,
  0xff,0x76,0x39,0xfc,0x38,0xd8,0x7f,0xfe,0x7f,0xfe,0x01,0xf8,0x03,0xfc,
  0x03,0xfe,0x0f,0xff,0x0f,0xff,0x0f,0xff,0xef,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xfe,0x3f,0xfc,0x3f,0xf8,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x05,0x55,0x55,0x50,0x00,0x00,0x00,
  0x00,0x55,0x55,0x55,0x55,0x00,0x00,0x00,0x05,0x55,0x05,0x55,0x05,0x50,
  0x00,0x00,0x55,0x55,0x05,0x55,0x05,0x55,0x00,0x00,0x55,0x55,0x55,0x55,
  0x55,0x55,0x00,0x00,0x55,0x55,0x55,0x55,0x55,0x55,0xee,0xe0,0x55,0x05,
  0x55,0x05,0x55,0x05,0xee,0xee,0x55,0x05,0x55,0x55,0x55,0x05,0xec,0xcf,
  0xce,0x55,0x55,0x55,0x55,0x55,0xec,0xff,0xce,0x55,0x55,0x55,0x55,0x55,
  0xef,0xff,0xfe,0x55,0x05,0x55,0x05,0x55,0xee,0xee,0xee,0x55,0x05,0x55,
  0x05,0x50,0x00,0xff,0xf0,0x05,0x55,0x55,0x55,0x00,0x00,0xff,0xf0,0x00,
  0x55,0x05,0x50,0x00,0xde,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xed,0xde,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xed,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,
  0xb0,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,
  0x00,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,
  0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0xb0,0xb0,0xb0,0xb0,
  0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xab,0xab,0xab,0x00,0xb0,0xb0,
  0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0x00,0xb0,0xab,0xab,0xab,0xab,
  0xb0,0xb0,0x00,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0xb0,0xab,0x2a,
  0x2a,0xfe,0x2a,0xab,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,
  0xab,0x2a,0xfe,0xfe,0x2a,0xab,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,
  0xb0,0xb0,0xab,0xfe,0xfe,0xfe,0xfe,0xab,0xb0,0xb0,0x00,0xb0,0xb0,0xb0,
  0x00,0xb0,0xb0,0xb0,0xab,0xab,0xab,0xab,0xab,0xab,0xb0,0xb0,0x00,0xb0,
  0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0xb0,
  0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,
  0x00,0x00,0xb0,0xb0,0x00,0xb0,0xb0,0x00,0x00,0x00,0xf9,0xfb,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0xf9,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xf9,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x00,0xec,0xa7,0x58,0x1f,0x38,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x00,0xec,0xb2,0x44,0xbf,0xb9,0xff,0xff,0x00,0x00,0x06,0x08,0x00,0xec,
  0xa7,0xa0,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x00,0x00,0x00,0x00,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x00,0x00,0x00,0x00,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x00,0x00,0x00,0x00
};
#endif /* sgi */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#ifdef  hpux
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* HPicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x66,0x66,0x00,0x5e,0x99,0x99,0x66,0x66,0x33,0x33,0x00,0x5f,
  0x99,0x99,0x66,0x66,0x00,0x00,0x00,0x60,0x99,0x99,0x33,0x33,0xff,0xff,
  0x00,0x61,0x99,0x99,0x33,0x33,0x06,0x48,0x50,0x69,0x63,0x6f,0x6e,0x48,
  0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,0x05,0x00,
  0x00,0xd0,0x00,0x80,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x05,0x00,0x00,0xd0,0x00,0x80,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x6d,0x26,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x00,0x00,0x00,0x00,0x00,0x6c,0x66,0x66,0xff,0xff,0xff,0xff,0x00,0x6d,
  0x66,0x66,0xff,0xff,0xcc,0xcc,0x00,0x6e,0x66,0x66,0xff,0xff,0x99,0x99,
  0x00,0x6f,0x66,0x66,0xff,0xff,0x66,0x66,0x00,0x70,0x66,0x66,0xff,0xff,
  0x33,0x33,0x00,0x71,0x66,0x66,0xff,0xff,0x00,0x00,0x00,0x72,0x66,0x66,
  0xcc,0xcc,0xff,0xff,0x00,0x73,0x66,0x66,0xcc,0xcc,0xcc,0xcc,0x00,0x74,
  0x66,0x66,0xcc,0xcc,0x99,0x99,0x00,0x75,0x66,0x66,0xcc,0xcc,0x66,0x66,
  0x00,0x76,0x66,0x66,0xcc,0xcc,0x33,0x33,0x00,0x77,0x66,0x66,0xcc,0xcc,
  0x00,0x00,0x00,0x78,0x66,0x66,0x99,0x99,0xff,0xff,0x00,0x79,0x66,0x66,
  0x99,0x99,0xcc,0xcc,0x00,0x7a,0x66,0x66,0x99,0x99,0x99,0x99,0x00,0x7b,
  0x66,0x66,0x99,0x99,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0xec,0xec,0xec,0xec,0x00,0x00,
  0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,
  0x7f,0xec,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0xec,0x7f,0x7f,0x7f,0x7f,0xec,0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0x7f,0x7f,0x7f,0x7f,0x7f,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,0x7f,0x7f,0xec,
  0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0x00,
  0xec,0x7f,0x7f,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0x00,0x00,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0x00,
  0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,
  0xec,0xec,0xec,0xec,0x7f,0x7f,0x7f,0xec,0x00,0x00,0x00,0x00,0x00,0xec,
  0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,0xec,0x7f,0x7f,0xec,0xec,0x00,0xec,
  0xec,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0xec,0xec,0x00,0x00,
  0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xec,0xec,0xec,0xec,
  0xec,0x00,0x00,0xec,0x7f,0x7f,0x7f,0xec,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0xec,0xec,0xfd,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,
  0xec,0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xec,0x7f,0x7f,0xec,0xfd,0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,0xec,0xfd,0x33,0xfd,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x7f,0x7f,0xec,0xff,0xff,0xff,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0x7f,0x7f,
  0xec,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xec,0xec,0xec,0xec,0xec,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,
  0x33,0xfd,0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xf9,
  0xfb,0xfb,0xfb,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0x33,0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,
  0xf9,0xf9,0xf7,0xf7,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0xf7,0xf7,0xf9,0xf9,0xfb,0xfb,0xfb,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0x00,0x00,
  0x00,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x02,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf5,0x55,0xf0,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xf5,0x5f,0xf0,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x55,0x5f,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0x55,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xf5,0x55,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x0f,0xf5,0x5f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x0f,0x55,0x5f,0xff,0xff,0x00,0x00,0x0f,0xff,0xff,
  0xff,0x00,0x00,0x00,0x00,0x00,0xff,0x55,0x55,0x55,0x5f,0x00,0x00,0xff,
  0x55,0x55,0x55,0xf0,0x00,0x00,0x00,0x00,0xf5,0x55,0xff,0x55,0x55,0xf0,
  0x00,0xf5,0x55,0x55,0x55,0x5f,0x00,0x00,0x00,0x0f,0xf5,0x5f,0xf0,0xf5,
  0x55,0xf0,0x0f,0xf5,0x5f,0xff,0x55,0x5f,0x00,0x00,0x00,0x0f,0x55,0x5f,
  0x00,0xf5,0x55,0xf0,0x0f,0x55,0x5f,0x00,0xf5,0x5f,0x00,0x00,0x00,0xff,
  0x55,0xff,0x0f,0xf5,0x5f,0xf0,0xff,0x55,0xff,0x00,0xf5,0x5f,0x00,0x00,
  0x00,0xf5,0x55,0xf0,0x0f,0x55,0x5f,0x00,0xf5,0x55,0xf0,0x0f,0x55,0x5f,
  0x00,0x00,0x0f,0xf5,0x5f,0xf0,0xff,0x55,0xff,0x0f,0xf5,0x5f,0xf0,0xff,
  0x55,0xff,0x00,0x00,0x0f,0x55,0x5f,0x00,0xf5,0x55,0xf0,0x0f,0x55,0x5f,
  0x00,0xf5,0x55,0xf0,0x00,0x00,0xff,0x55,0xff,0x0f,0xf5,0x5f,0xf0,0xff,
  0x55,0xff,0x0f,0xf5,0x5f,0xf0,0x00,0x00,0xf5,0x55,0xf0,0x0f,0x55,0x5f,
  0x00,0xf5,0x55,0xff,0xff,0x55,0x5f,0x00,0x00,0x0f,0xf5,0x5f,0xf0,0xff,
  0x55,0xff,0x0f,0xf5,0x55,0x55,0x55,0x55,0xff,0x00,0x00,0x0f,0xff,0xff,
  0x00,0xff,0xff,0xf0,0x0f,0x55,0x5f,0xff,0xff,0xff,0xf0,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x55,0xff,0xa0,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x55,0xfb,0xa0,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf5,0x5f,0xab,0xa0,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xf5,0x5f,
  0xab,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,
  0x55,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0x55,0xff,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xff,0xff,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xba,0xba,0x00,0x00,
  0x00,0x00,0xdd,0xee,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xab,0xba,0xbb,
  0xaa,0xae,0xee,0xdd,0xcc,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,
  0xa0,0xab,0xbb,0xbb,0xbb,0xcc,0xdd,0xee,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,
  0xaa,0xaa,0x00,0x0a,0xaa,0xae,0xee,0xdd,0x00,0x00,0x01,0x00,0x00,0x0f,
  0x80,0x00,0x00,0x08,0x80,0x00,0x00,0x19,0x80,0x00,0x00,0x11,0x00,0x00,
  0x00,0x33,0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x66,0x00,0x00,0x00,0x47,
  0xc1,0xfc,0x00,0xc0,0x43,0x02,0x00,0x8c,0x22,0x01,0x01,0x9a,0x26,0x71,
  0x01,0x12,0x24,0x49,0x03,0x36,0x6c,0xc9,0x02,0x24,0x48,0x91,0x06,0x6c,
  0xd9,0xb3,0x04,0x48,0x91,0x22,0x0c,0xd9,0xb3,0x66,0x08,0x91,0x23,0xc4,
  0x19,0xb3,0x60,0x0c,0x1f,0x3e,0x47,0xf8,0x00,0x00,0x4e,0x00,0x00,0x00,
  0xca,0x00,0x00,0x00,0x9a,0x00,0x00,0x01,0x9a,0x00,0x00,0x01,0x3f,0x00,
  0x00,0x03,0x31,0x00,0x00,0x03,0xf1,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,
  0x15,0x00,0xbf,0xff,0xe4,0xfd,0x00,0x00,0x0a,0x00,0xbf,0xff,0xf1,0xfd,
  0x00,0x0f,0x80,0x00,0x00,0x0f,0x80,0x00,0x00,0x1f,0x80,0x00,0x00,0x1f,
  0x00,0x00,0x00,0x3f,0x00,0x00,0x00,0x3e,0x00,0x00,0x00,0x7e,0x00,0x00,
  0x00,0x7f,0xc1,0xfc,0x00,0xff,0xc3,0xfe,0x00,0xff,0xe3,0xff,0x01,0xfb,
  0xe7,0xff,0x01,0xf3,0xe7,0xcf,0x03,0xf7,0xef,0xcf,0x03,0xe7,0xcf,0x9f,
  0x07,0xef,0xdf,0xbf,0x07,0xcf,0x9f,0x3e,0x0f,0xdf,0xbf,0x7e,0x0f,0x9f,
  0x3f,0xfc,0x1f,0xbf,0x7f,0xfc,0x1f,0x3e,0x7f,0xf8,0x00,0x00,0x7e,0x00,
  0x00,0x00,0xfe,0x00,0x00,0x00,0xfe,0x00,0x00,0x01,0xfe,0x00,0x00,0x01,
  0xff,0x00,0x00,0x03,0xff,0x00,0x00,0x03,0xff,0x00,0x00,0x00,0x1f,0x00,
  0x00,0x00,0x1f,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,
  0xf1,0xff,0x00,0x00,0x00,0x40,0x03,0x80,0x07,0x80,0x05,0x00,0x0f,0x9e,
  0x0a,0xd1,0x1f,0x7f,0x17,0xef,0x3e,0xfd,0x2f,0xdf,0x7f,0xfe,0x00,0xb0,
  0x01,0xf0,0x01,0x70,0x01,0xf0,0xff,0xff,0xff,0xff,0x03,0x80,0x07,0x80,
  0x07,0x00,0x0f,0x9e,0x0f,0xdf,0x1f,0xff,0x1f,0xff,0x3f,0xff,0x3f,0xff,
  0x7f,0xfe,0x00,0xf0,0x01,0xf0,0x01,0xf0,0x01,0xf0,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x66,0x60,0x00,0x00,0x00,0x00,0x00,
  0x06,0x66,0x60,0x00,0x00,0x00,0x00,0x00,0x06,0x06,0x00,0x00,0x00,0x00,
  0x00,0x00,0x66,0x66,0x60,0x06,0x66,0x60,0x00,0x00,0x60,0x60,0x66,0x06,
  0x00,0x06,0x00,0x06,0x66,0x66,0x06,0x66,0x66,0x66,0x00,0x06,0x06,0x66,
  0x66,0x60,0x66,0x66,0x00,0x66,0x66,0x60,0x66,0x66,0x66,0x06,0x00,0x60,
  0x66,0x66,0x66,0x06,0x66,0x66,0x06,0x66,0x66,0x66,0x66,0x66,0x66,0x60,
  0x00,0x00,0x00,0x00,0x60,0x6f,0x00,0x00,0x00,0x00,0x00,0x06,0x66,0xff,
  0x00,0x00,0x00,0x00,0x00,0x06,0x06,0xff,0x00,0x00,0x00,0x00,0x00,0x06,
  0x66,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xec,0x00,0xec,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xec,
  0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0xec,0x00,0xec,0x00,0xec,0xec,
  0x00,0xec,0x00,0x00,0x00,0xec,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,
  0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0x00,0xec,0x00,0xec,
  0xec,0xec,0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0x00,0x00,0xec,0xec,
  0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0xec,0x00,0x00,
  0xec,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0xec,0xec,0xec,0xec,0xec,
  0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0x00,0xec,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,
  0x00,0xec,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xec,0xec,0xec,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x00,0xec,0xa7,0x58,0x1f,0x96,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x38,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x34,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0xec,0x9e,0x9c,0xbf,0xb9,0xff,0xff,0x00,0x00,0x04,0x04,
  0x00,0x00,0x00,0x00,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x00,0xec,
  0x94,0x14,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x00,0x00,0x00,0x00,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x00,0x00,0x00,0x00,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x00,0x00,0x00,0x00
};
#endif /* hpux */
#endif /* HOST_ICON */

#ifndef HOST_ICON
#ifdef  linux
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* LINUXicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x66,0xff,0x0c,0x0c,0x0c,0x0f,0x00,0x00,0x00,0x0f,0xc0,0xc0,
  0xc0,0xcf,0x60,0x66,0x00,0x0f,0xc0,0xc0,0xc0,0xcf,0x00,0x00,0x00,0x0f,
  0x0c,0x0c,0x0c,0x0f,0x66,0x00,0x09,0x4c,0x49,0x4e,0x55,0x58,0x69,0x63,
  0x6f,0x6e,0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,
  0x05,0x00,0x00,0x68,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x05,0x00,0x00,0x68,0x01,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x39,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0xcc,0xcc,0xcc,0xc3,0xcf,0xc0,0xc0,0xcf,0x00,0x00,0x00,0x0f,0x0c,0x0c,
  0x0f,0xcc,0xcc,0xcc,0xcc,0xcc,0xcf,0x0c,0x0c,0x0f,0x00,0x00,0x00,0x0f,
  0xc0,0xc0,0xcf,0xff,0xff,0xff,0xff,0xff,0xff,0xc0,0xc0,0xcf,0x00,0x00,
  0x00,0x0f,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0f,
  0x00,0x00,0x00,0x0f,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,
  0xc0,0xcf,0x00,0x00,0x00,0x0f,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,
  0x0c,0x0c,0x0c,0x0f,0x00,0x00,0x00,0x0f,0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,
  0xc0,0xc0,0xc0,0xc0,0xc0,0xcf,0x00,0x00,0x00,0x0f,0x0c,0x0c,0x0c,0x0c,
  0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0f,0x00,0x00,0x00,0x0f,0xc0,0xc0,
  0xc0,0xc0,0xc0,0xc0,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xf0,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x22,0xff,0xfc,0xcc,0xcf,
  0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0xff,0xcc,
  0xff,0xcc,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0xcc,0xff,0xcc,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xfc,0xcc,0xcf,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xfc,0xcc,0xcf,0xff,0xff,0xf0,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xcf,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
  0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xf3,0x78,0xf3,
  0x78,0xf3,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x0e,0xee,0xe0,0x00,0x00,0x00,0x00,0x00,
  0x0f,0xcc,0xff,0xff,0xff,0xf7,0xf8,0xf0,0xe7,0x77,0x7e,0x00,0x00,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0xee,0xee,0xee,0xee,
  0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x0f,0xff,0xe7,0x77,
  0x77,0x77,0x77,0x7e,0x0f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0xf0,
  0xe7,0x77,0x77,0x7f,0x77,0x7e,0x00,0x0f,0xff,0xff,0xff,0xff,0xff,0xff,
  0xf0,0x0f,0xe7,0x77,0x77,0xf7,0x77,0x7e,0x00,0x00,0xfc,0xcc,0xff,0xff,
  0xff,0xff,0xf0,0x00,0xe7,0x77,0x7f,0x77,0x77,0x7e,0x00,0x00,0xfc,0xcc,
  0xff,0xff,0xff,0xff,0xf0,0x00,0xe7,0x77,0xf7,0x77,0x77,0x7e,0x00,0x00,
  0xfc,0xcc,0xff,0xff,0xff,0xff,0xf0,0x00,0xe7,0x7f,0x77,0xf7,0xf7,0x7e,
  0x00,0x00,0xfc,0xcc,0xff,0xff,0xff,0xff,0xf0,0x00,0xe7,0x77,0x77,0x77,
  0x77,0x7e,0x00,0x00,0xfc,0xcc,0xff,0xff,0xff,0xff,0xf0,0x00,0xee,0xee,
  0xee,0xee,0xee,0xee,0x00,0x00,0xfc,0xcc,0xff,0xff,0xff,0xff,0xf0,0x00,
  0x00,0x00,0x0a,0xba,0x00,0x00,0x00,0x00,0x0f,0xcc,0xff,0xff,0xff,0xff,
  0xf0,0x00,0x00,0x00,0xff,0xff,0xf0,0x00,0x00,0x00,0x0f,0xcc,0xcf,0xff,
  0xff,0xff,0xf0,0x00,0x00,0x00,0xf3,0x33,0xf0,0x00,0x00,0x00,0x00,0xfc,
  0xcc,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xf0,0x00,0x00,0x00,
  0x0f,0xff,0xff,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0xab,0xab,0xa0,0x00,
  0x00,0x0f,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xde,0xea,0xbb,0xab,
  0xba,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcb,0xbb,
  0xba,0x0a,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xde,0xea,0xa0,0x00,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x17,0x17,0xff,0xff,0xff,0x2b,
  0x2b,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x17,
  0xff,0xff,0x2b,0x2b,0xff,0xff,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xff,0x2b,0x2b,0xff,0xff,0x2b,0x2b,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,0x2b,
  0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0x2b,0x2b,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xd8,
  0xc0,0xe3,0xff,0xd8,0xc0,0xe3,0xff,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xc0,0xff,0xe3,0xff,0x00,0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0xab,0xab,0xab,0xab,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0xff,0xff,0xff,
  0xab,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0x00,0xff,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xff,0x2a,0x2a,
  0x2a,0xab,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0xff,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,0x2b,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xab,0x2a,
  0x2a,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,
  0xff,0x2b,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,
  0x00,0x00,0xab,0x2a,0x2a,0x2a,0xff,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,
  0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0xab,0x2a,0x2a,0xff,0x2a,0x2a,0xff,0x2a,
  0xff,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,0x2b,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0x00,0x00,0xff,0x2b,
  0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,
  0x00,0x00,0xff,0x2b,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x2b,0x2b,
  0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0x2b,0x2b,0x2b,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,
  0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xf9,0xfb,0xfb,0xfd,
  0x33,0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf7,0x33,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf7,0xf7,0xf9,0xfb,0xfb,0xfd,0xfd,0x00,0x00,0x00,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x0f,0xc0,0x00,0x00,0x3f,0xe0,0x00,0x00,0xff,0xf0,
  0x00,0x07,0xe1,0xf0,0x00,0x03,0xcc,0xf8,0x00,0x00,0xcc,0xf8,0x00,0x00,
  0x21,0xf8,0x00,0x00,0x21,0xf8,0x00,0x00,0x17,0xf0,0x00,0x00,0x3f,0xf8,
  0x00,0x00,0x62,0x28,0x00,0x00,0x3f,0xfc,0x78,0x00,0x4f,0xea,0x84,0x3f,
  0xff,0xfe,0xff,0xff,0xff,0xf7,0x80,0x17,0xff,0xfa,0x81,0x11,0xff,0xf9,
  0x82,0x10,0x8f,0xf8,0x84,0x10,0x8f,0xf8,0x88,0x10,0x8f,0xf8,0x92,0x90,
  0x8f,0xf8,0x80,0x10,0x8f,0xf8,0xff,0xf0,0x8f,0xf8,0x05,0x00,0x4f,0xf8,
  0x0f,0x80,0x47,0xf8,0x08,0x80,0x23,0xf0,0x0f,0x80,0x7f,0xe0,0x0a,0x81,
  0xff,0xf0,0xf2,0x7f,0xff,0xff,0x05,0x00,0x00,0x00,0xf8,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xc0,0x00,0x00,0x3f,0xe0,0x00,0x00,
  0xff,0xf0,0x00,0x07,0xff,0xf0,0x00,0x03,0xff,0xf8,0x00,0x00,0xff,0xf8,
  0x00,0x00,0x3f,0xf8,0x00,0x00,0x3f,0xf8,0x00,0x00,0x1f,0xf0,0x00,0x00,
  0x3f,0xf8,0x00,0x00,0x7f,0xf8,0x00,0x00,0x3f,0xfc,0x78,0x00,0x7f,0xfe,
  0xfc,0x3f,0xff,0xfe,0xff,0xff,0xff,0xff,0xff,0xf7,0xff,0xfa,0xff,0xf1,
  0xff,0xf9,0xff,0xf0,0xff,0xf8,0xff,0xf0,0xff,0xf8,0xff,0xf0,0xff,0xf8,
  0xff,0xf0,0xff,0xf8,0xff,0xf0,0xff,0xf8,0xff,0xf0,0xff,0xf8,0x07,0x00,
  0x7f,0xf8,0x0f,0x80,0x7f,0xf8,0x0f,0x80,0x3f,0xf0,0x0f,0x80,0x7f,0xe0,
  0x0f,0x81,0xff,0xf0,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xf8,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0x00,0x38,0x00,0xfc,0x03,0xde,0x00,0xfe,
  0x00,0x7e,0x00,0xd6,0xe0,0xff,0xff,0xff,0xff,0xff,0xfc,0xbe,0xfc,0xbe,
  0xfc,0xbe,0x38,0xbe,0x38,0xfc,0xff,0xfe,0xff,0xfe,0x00,0x38,0x00,0xfc,
  0x03,0xfe,0x00,0xfe,0x00,0x7e,0x00,0xfe,0xe0,0xff,0xff,0xff,0xff,0xff,
  0xfc,0xfe,0xfc,0xfe,0xfc,0xfe,0x38,0xfe,0x38,0xfc,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0x00,0xff,0xf0,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x22,0xff,0x0f,0xff,0xf0,
  0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x0f,0xff,
  0xff,0xf0,0x00,0x00,0x00,0x00,0x38,0x73,0x87,0xf0,0xee,0xe0,0x00,0x00,
  0xff,0xff,0xff,0xf8,0xee,0xee,0xef,0xff,0xff,0xff,0xff,0xf3,0xe7,0x7f,
  0x7e,0xff,0xff,0xff,0xff,0xf7,0xe7,0xff,0x7e,0x00,0xfc,0xff,0xff,0xf0,
  0xef,0xff,0xfe,0x00,0xfc,0xff,0xff,0xf0,0xee,0xee,0xee,0x00,0xfc,0xff,
  0xff,0xf0,0x00,0xff,0xf0,0x00,0xfc,0xff,0xff,0xf0,0x00,0xff,0xf0,0x00,
  0xff,0xff,0xff,0x00,0xee,0xbb,0xba,0xaf,0xff,0xff,0xff,0xed,0xee,0xba,
  0xbb,0xbb,0xbb,0xbb,0xbe,0xed,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x17,0xff,0xff,0x00,0xff,0xff,0xff,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xd8,0xe3,0xc0,0xd8,0xe3,0xc0,0xff,0x00,0xfc,0xfc,0xfc,0x00,0x00,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xfc,0xfc,0xfc,0xfc,
  0xfc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd8,0xfc,0xc0,
  0xc0,0xff,0xc0,0xfc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc0,
  0xfc,0xc0,0xff,0xff,0xc0,0xfc,0x00,0x00,0xff,0x2b,0xff,0xff,0xff,0xff,
  0xff,0x00,0xfc,0xff,0xff,0xff,0xff,0xfc,0x00,0x00,0xff,0x2b,0xff,0xff,
  0xff,0xff,0xff,0x00,0xfc,0xfc,0xfc,0xfc,0xfc,0xfc,0x00,0x00,0xff,0x2b,
  0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,
  0xff,0x2b,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x00,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xfc,0xfc,0x5e,0x5e,
  0x5e,0x89,0x89,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xf9,0xfc,0xfc,
  0x5e,0x89,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0xfc,0xfc,0xf9,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x00,0xec,0xb8,0x74,0x1b,0x8c,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0xec,0xb6,0x4c,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x00,0xec,0xb6,0xc4,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x00,0x00,
  0x00,0x00,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x00,0x00,0x00,0x00,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x00,0xec,0xb6,0x80,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x00,0xec,0xb6,0x3c
};
#endif /* linux */
#endif /* HOST_ICON */
#endif /* USE_HOST_ICON */

/* default for System V */

#ifndef HOST_ICON
#ifdef  SYSTYPE_SYSV
#define HOST_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* AT&Ticon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x08,0x41,0x54,0x26,0x54,0x69,0x63,0x6f,
  0x6e,0x26,0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,
  0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x39,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x06,0x66,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,0x09,0x99,
  0x09,0x99,0x08,0xc8,0x09,0x99,0x09,0x99,0x0b,0x68,0x08,0xc8,0x08,0xc8,
  0x08,0xc8,0x09,0x99,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x03,0x33,
  0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
  0x30,0x00,0x00,0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x33,0x33,0x33,0x33,0x33,0x00,0x00,0x33,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x33,0x33,0x33,0x30,0x00,0x00,
  0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,
  0x00,0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x00,
  0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x33,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x33,0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x00,0x00,0x00,0x03,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x30,0x00,0x00,0x00,0x30,0x00,0x00,0x03,0x30,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x30,0x00,0x00,
  0x33,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x33,
  0x00,0x00,0x30,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x30,0x00,0x00,
  0x00,0x03,0x00,0x00,0x33,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
  0x33,0x33,0x33,0x33,0x00,0x00,0x33,0x33,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x33,0x33,0x33,0x33,0x33,0x00,0x00,0x30,0x00,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x03,0x00,0x00,0x33,0x33,0x30,0x00,
  0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,0x00,0x00,0x33,0x33,
  0x33,0x30,0x00,0x00,0x00,0x00,0x33,0x33,0x33,0x33,0x33,0x33,0x00,0x00,
  0x33,0x00,0x00,0x03,0x33,0x33,0x33,0x30,0x00,0x00,0x00,0x00,0x00,0x33,
  0x00,0x00,0x33,0x33,0x33,0x33,0x30,0x00,0x03,0x33,0x33,0x33,0x33,0x33,
  0x33,0x3a,0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x3a,0x00,0x00,0x03,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x03,0xda,0x00,0x00,0x00,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xda,0x00,0x00,0x00,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x3a,0xda,0x00,0x00,0x00,0x00,
  0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x0a,0xda,0x00,0x00,
  0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x30,0x0a,0xda,
  0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x00,
  0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x03,0x33,0x00,0x00,0x00,0x03,0x33,
  0x00,0x00,0xf3,0x33,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0x33,0x33,
  0x30,0x00,0x00,0x00,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xab,0xab,0xa0,0x00,0xdd,0xee,0xea,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xbb,0xab,0xbe,0xed,0xcc,0xbb,
  0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xba,0x0a,0xbb,0xbc,
  0xdd,0xee,0xea,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xa0,0x00,
  0xae,0xed,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0x00,0x00,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,
  0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0xd8,0x00,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,0xd8,0xd8,
  0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,
  0x00,0x00,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,
  0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0x00,0x00,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xfd,0x00,0x00,0x00,0x00,0x00,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xfd,0x00,0x00,
  0x00,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,
  0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xfd,0x33,0xfd,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0x00,0xfd,0x33,0xfd,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,
  0x00,0xfd,0x33,0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xfd,0x33,0xfd,0x33,0xfd,0x00,0x00,0x00,0xf9,0xf9,0xfb,0xfb,
  0xfb,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0x33,0x33,0xfd,0x33,0x33,0xfb,0xfb,0xf9,
  0xf7,0xf7,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xfd,0x00,0xfd,
  0x33,0x33,0x33,0xf7,0xf9,0xf9,0xfb,0xfb,0xfb,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0x00,0x00,0x00,0xfd,0xfb,0xfb,0xf9,0x00,0x00,0x01,0x00,0x00,0x7f,
  0xc0,0x00,0x01,0x83,0xf0,0x00,0x03,0xff,0x0c,0x00,0x04,0x00,0xfe,0x00,
  0x08,0x00,0x7f,0x00,0x1f,0xff,0xc0,0x80,0x30,0x00,0x3f,0xc0,0x20,0x00,
  0x1f,0xc0,0x7f,0xff,0xe0,0x20,0x60,0x00,0x1f,0xe0,0xe0,0x00,0x1f,0xf0,
  0xbf,0xff,0xe0,0x10,0xe0,0x00,0x1f,0xf0,0xf0,0x00,0x3f,0xf0,0x8f,0xff,
  0xc0,0x10,0xf8,0x00,0x7f,0xf0,0xfe,0x00,0xff,0xf0,0xc1,0xfe,0x00,0x30,
  0xff,0x87,0xff,0xf0,0x7f,0xff,0xff,0xf0,0x60,0x00,0x00,0x50,0x3f,0xff,
  0xff,0xd0,0x3f,0xff,0xff,0xd0,0x08,0x00,0x01,0x50,0x07,0xff,0xfe,0x50,
  0x07,0xff,0xfc,0xf8,0x01,0xc0,0x70,0x88,0x00,0x3f,0x80,0xf8,0x00,0x00,
  0x00,0xa8,0xff,0xff,0xff,0x27,0x00,0x00,0x00,0x50,0xff,0xff,0xff,0x8f,
  0x00,0x7f,0xc0,0x00,0x01,0xff,0xf0,0x00,0x03,0xff,0xfc,0x00,0x07,0xff,
  0xfe,0x00,0x0f,0xff,0xff,0x00,0x1f,0xff,0xff,0x80,0x3f,0xff,0xff,0xc0,
  0x3f,0xff,0xff,0xc0,0x7f,0xff,0xff,0xe0,0x7f,0xff,0xff,0xe0,0xff,0xff,
  0xff,0xf0,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xf0,
  0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xf0,0xff,0xff,
  0xff,0xf0,0xff,0xff,0xff,0xf0,0x7f,0xff,0xff,0xf0,0x7f,0xff,0xff,0xf0,
  0x3f,0xff,0xff,0xf0,0x3f,0xff,0xff,0xf0,0x1f,0xff,0xfe,0x70,0x0f,0xff,
  0xfe,0x70,0x07,0xff,0xfc,0x70,0x03,0xff,0xf0,0xf8,0x00,0x3f,0x80,0xf8,
  0x00,0x00,0x00,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdf,0xff,0xff,
  0xff,0x8f,0x00,0x00,0x00,0x40,0x1f,0xc0,0x3b,0xf0,0x7f,0xf8,0x60,0xf8,
  0xff,0xfc,0xc0,0x7c,0xff,0xfc,0xe0,0xfc,0xff,0xfc,0xfb,0xfc,0xff,0xfc,
  0x7f,0xfc,0x3f,0xfe,0x1f,0xce,0x7f,0xff,0x7f,0xff,0x1f,0xc0,0x3f,0xf0,
  0x7f,0xf8,0x7f,0xf8,0xff,0xfc,0xff,0xfc,0xff,0xfc,0xff,0xfc,0xff,0xfc,
  0xff,0xfc,0xff,0xfc,0x7f,0xfc,0x3f,0xfc,0x1f,0xfe,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x03,0x33,0x33,0x33,0x00,0x00,0x00,0x00,0x33,
  0x30,0x33,0x33,0x33,0x00,0x00,0x03,0x33,0x33,0x33,0x33,0x33,0x30,0x00,
  0x03,0x30,0x00,0x00,0x33,0x33,0x30,0x00,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x00,0x33,0x00,0x00,0x00,0x03,0x33,0x33,0x00,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x00,0x33,0x30,0x00,0x00,0x33,0x33,0x33,0x00,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x00,0x33,0x33,0x30,0x33,0x33,0x33,0x3a,0x00,
  0x33,0x33,0x33,0x33,0x33,0x33,0x3a,0x00,0x03,0x33,0x33,0x33,0x33,0x33,
  0xaa,0x00,0x00,0x33,0x33,0x33,0x33,0x33,0xff,0xf0,0x00,0x03,0x33,0x33,
  0x33,0x00,0xff,0xf0,0xde,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xde,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xd8,0xd8,0xd8,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,
  0x00,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,
  0xd8,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xd8,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xd8,0xd8,0xd8,0x00,
  0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,
  0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xfd,
  0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xfd,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xfd,0xfd,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xff,0xff,0xff,0x00,0xf9,0xfb,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xf9,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xfb,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xf6,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xef,0x04,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x03,0x71,0xea,0xe4,0xbf,0xb9,0xff,0xff,0x20,0x00,0x06,0x08,0x03,0x71,
  0xf5,0xdc,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xf1,0xc4,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x03,0x71,0xf3,0x8c,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x03,0x71,0xf2,0x90
};
#endif  /* SYSTYPE_SYSV */
#endif  /* HOST_ICON */

/* default */

#ifndef HOST_ICON
#define BSD_ICON 1
unsigned char color_cap_icon[ICON_RSRC_LEN] = { /* BSDicon */
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x00,0x60,0x00,0x03,0x00,0x70,0x00,0x30,0x06,0x4a,0x80,0x66,
  0x06,0x70,0x01,0x60,0x00,0x02,0xf2,0x24,0x6b,0x1d,0x8e,0x2d,0x6b,0x00,
  0x20,0xff,0xf4,0x42,0x44,0x42,0x08,0x53,0x55,0x4e,0x20,0x69,0x63,0x6f,
  0x6e,0x4e,0x02,0x00,0x00,0x00,0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,
  0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x72,0x73,
  0x72,0x63,0x52,0x53,0x45,0x44,0x01,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xb0,0x0e,0x36,0x73,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x6e,
  0x00,0x1c,0x14,0x70,0x00,0x10,0x06,0x0c,0x40,0x00,0x2e,0x67,0x00,0x00,
  0x84,0x70,0x00,0x10,0x06,0x0c,0x40,0x00,0xe5,0x67,0x78,0x70,0x00,0x10,
  0x06,0x4a,0x80,0x67,0x7e,0x70,0x10,0xc0,0x2c,0x00,0x0b,0x67,0x34,0x4a,
  0x44,0x67,0x06,0x70,0x00,0x60,0x00,0x02,0x64,0x48,0x7a,0x02,0x76,0x2f,
  0x0c,0x4e,0xba,0x8b,0x20,0x4a,0x00,0x50,0x4f,0x67,0x14,0x70,0x00,0x10,
  0x2c,0x00,0x1b,0xe1,0x88,0x72,0x00,0x12,0x2c,0x00,0x1a,0x38,0x01,0xd8,
  0x40,0x60,0x3a,0x70,0x00,0x60,0x00,0x02,0x3a,0x4a,0x43,0x67,0x06,0x70,
  0x00,0x60,0x00,0x02,0x30,0x48,0x7a,0x02,0x36,0x2f,0x0c,0x4e,0xba,0x8a,
  0xec,0x4a,0x00,0x50,0x4f,0x67,0x14,0x70,0x00,0x10,0x2c,0x00,0x1b,0xe1,
  0x88,0x72,0x00,0x12,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0x00,0x00,0x00,0x00,0x0f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x0f,0xf0,0x00,0xff,0xff,0xf0,0x00,0xff,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x0f,0x3f,0x0f,0x33,0x33,0x3f,0x0f,0x3f,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x0f,0x33,0xf3,0x3f,0xff,0x33,0xf3,0x3f,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x3f,0xff,0x33,0xf3,0x3f,
  0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x00,0xff,0x00,0xf3,
  0x3f,0x33,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0x00,0xf0,
  0x00,0x0f,0x3f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
  0xf0,0xff,0xf0,0x0f,0x3f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xf0,0xff,0xf0,0x0f,0x3f,0x33,0x3f,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0xf0,0xff,0xf0,0x0f,0x3f,0x33,0x3f,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xf0,0x00,0xf0,0x00,0x0f,0x3f,0x33,0x3f,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0x00,0xf0,0x00,0xf3,0x3f,0x33,
  0x3f,0x00,0x0e,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0xff,0x3f,
  0xf3,0x33,0x3f,0x00,0xe0,0x0e,0x0e,0xee,0xe0,0x00,0x00,0x00,0xf3,0x33,
  0xf3,0x33,0x33,0x33,0x3f,0x00,0xe0,0xe0,0xe7,0x77,0x7e,0x00,0x00,0x00,
  0xf3,0x33,0xf3,0x3f,0xff,0x33,0xf0,0x00,0xee,0x00,0xee,0xee,0xee,0xee,
  0xee,0xee,0x0f,0xff,0x33,0x33,0xf3,0x33,0xf0,0x0e,0xe0,0x0e,0xe7,0x77,
  0x77,0x77,0x77,0x7e,0x00,0xf3,0x33,0x3f,0x33,0x3f,0xee,0xe0,0x0e,0xe0,
  0xe7,0x77,0x77,0x7f,0x77,0x7e,0x00,0x0f,0xff,0xff,0xff,0xf3,0xf0,0x00,
  0x00,0x00,0xe7,0x77,0x77,0xf7,0x77,0x7e,0x00,0xf3,0x33,0x33,0xf3,0x33,
  0xf0,0x00,0x0f,0xf0,0xe7,0x77,0x7f,0x77,0x77,0x7e,0x00,0xff,0xf3,0x3f,
  0x33,0x33,0xf0,0x00,0xf0,0x0f,0xe7,0x77,0xf7,0x77,0x77,0x7e,0x0f,0x33,
  0x3f,0xef,0x33,0x3f,0xf0,0x00,0xf0,0x0f,0xe7,0x7f,0x77,0xf7,0xf7,0x7e,
  0x0f,0x33,0x3f,0x33,0xff,0xf3,0xf0,0x0f,0x00,0xf0,0xe7,0x77,0x77,0x77,
  0x77,0x7e,0x0f,0xf3,0x3f,0x33,0x33,0x33,0xf0,0xf0,0x0f,0x00,0xee,0xee,
  0xee,0xee,0xee,0xee,0x00,0xef,0xf3,0x3f,0x33,0x33,0xff,0x00,0x0f,0x00,
  0x00,0x00,0x0a,0xca,0x00,0x00,0x0e,0xf3,0x33,0x33,0x33,0x3f,0xf0,0x00,
  0xf0,0x00,0x00,0x00,0xaa,0xaa,0xa0,0x00,0xe0,0x0f,0x33,0x33,0x33,0x3f,
  0x00,0xf0,0xf0,0xf0,0x00,0x00,0xa3,0x33,0xa0,0x00,0x00,0x00,0xf3,0x3f,
  0x33,0x3f,0x00,0x0f,0xff,0x00,0x00,0x00,0xaa,0xaa,0xa0,0x00,0x00,0x00,
  0xf3,0x3f,0x33,0x3f,0x00,0x00,0xf0,0x00,0x00,0x00,0xab,0xab,0xa0,0x00,
  0x00,0x00,0xf3,0x3f,0x33,0x3f,0x00,0x00,0x00,0x00,0xdd,0xee,0xbb,0xab,
  0xba,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,0xee,0xdd,0xcc,0xbb,
  0xba,0x0a,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xcc,
  0xdd,0xee,0xa0,0x00,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xae,
  0xee,0xdd,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xd8,0xff,
  0x00,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0xff,0xd8,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xff,0xff,0xd8,0xd8,0xff,0xd8,
  0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xd8,0xff,0xff,0xff,0xd8,0xd8,
  0xff,0xd8,0xd8,0xff,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,0xff,0xff,
  0x00,0x00,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,
  0x00,0x00,0xff,0x00,0x00,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xff,0xfe,0xfe,0x00,0xff,0xfe,0xfe,0x00,0x00,0xff,0xd8,0xff,
  0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xfe,0xfe,0x00,0xff,0xfe,0xfe,0x00,
  0x00,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xfe,0xfe,0x00,
  0xff,0xfe,0xfe,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0xff,0xd8,0xff,0xd8,0xd8,
  0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0xff,0xd8,
  0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0xfc,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xd8,0xff,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0x00,0xfc,0x00,
  0x00,0xfc,0x00,0xab,0xab,0xab,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,
  0x00,0x00,0xfc,0x00,0xfc,0x00,0xab,0x54,0x54,0x54,0x54,0xab,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xff,0xff,
  0xd8,0xd8,0xff,0x00,0x00,0x00,0xfc,0xfc,0x00,0x00,0xab,0xab,0xab,0xab,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0xff,0xff,0xff,0xd8,0xd8,
  0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0xfc,0xfc,0x00,0x00,0xfc,
  0xab,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,
  0xff,0xd8,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xfc,0xfc,0xfc,0xfc,0x00,
  0x00,0xfc,0xfc,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xfe,0x2a,0x2a,
  0x2a,0xab,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd8,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x2a,0x2a,0x2a,0x2a,0x2a,
  0xfe,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0x00,0xab,0x2a,
  0x2a,0x2a,0x2a,0xfe,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0x00,0xff,0xff,
  0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0xff,0x00,
  0x00,0xff,0xab,0x2a,0x2a,0x2a,0xfe,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,
  0x00,0xff,0xd8,0xd8,0xd8,0xff,0xfc,0xff,0xd8,0xd8,0xd8,0xff,0xff,0x00,
  0x00,0x00,0xff,0x00,0x00,0xff,0xab,0x2a,0x2a,0xfe,0x2a,0x2a,0xfe,0x2a,
  0xfe,0x2a,0x2a,0xab,0x00,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xff,0xff,
  0xff,0xd8,0xff,0x00,0x00,0xff,0x00,0x00,0xff,0x00,0xab,0x2a,0x2a,0x2a,
  0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0x2a,0xab,0x00,0xff,0xff,0xd8,0xd8,0xff,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,0x00,0xff,0x00,0x00,0xff,0x00,0x00,
  0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0x00,
  0xfc,0xff,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xd8,0xff,0xff,0x00,0x00,
  0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0xf8,0xfd,0x00,0x00,
  0x00,0x00,0x00,0xfc,0xff,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xff,
  0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
  0xff,0xff,0xff,0x00,0x00,0x00,0xfc,0x00,0x00,0xff,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xff,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0x00,0x00,0x00,
  0x00,0x00,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0xff,0xff,0xff,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xff,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,
  0x00,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfd,0x33,0xfd,0x33,
  0xfd,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xd8,0xd8,0xff,0xd8,0xd8,
  0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xf9,0xfb,0xfb,
  0x33,0x33,0xfd,0x33,0x33,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,
  0xf6,0xf6,0x33,0x33,0x33,0xfd,0x00,0xfd,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0xf6,0xf6,0xf9,0xf9,0xfb,0xfb,0xfd,0x00,0x00,0x00,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfb,0xfb,0xfb,0xf9,0xf9,0x00,0x00,0x01,0x00,0x00,0x0c,
  0x01,0x80,0x00,0x18,0xf8,0xc0,0x00,0x15,0x05,0x40,0x00,0x12,0x72,0x40,
  0x00,0x0d,0xc9,0x40,0x00,0x13,0x24,0xc0,0x00,0x22,0x14,0x40,0x00,0x3b,
  0x94,0x40,0x00,0x3b,0x94,0x40,0x00,0x3b,0x94,0x40,0x00,0x22,0x14,0x40,
  0x00,0x22,0x24,0x44,0x00,0x1f,0xd8,0x49,0x78,0x08,0x80,0x4a,0x84,0x08,
  0x9c,0x8c,0xff,0xf7,0x08,0x99,0x80,0x12,0x11,0xe6,0x81,0x11,0xfe,0x80,
  0x82,0x12,0x08,0x86,0x84,0x13,0x90,0x89,0x88,0x14,0x71,0x89,0x92,0x94,
  0x4e,0x92,0x80,0x16,0x40,0xa4,0xff,0xf3,0x90,0xc4,0x05,0x06,0x01,0x88,
  0x0f,0x89,0x01,0x2a,0x08,0x80,0x91,0x1c,0x0f,0x80,0x91,0x08,0x0a,0x80,
  0x91,0x00,0xf2,0x7f,0xff,0xff,0x05,0x00,0x00,0x00,0xf8,0xff,0xff,0xff,
  0x00,0x0c,0x01,0x80,0x00,0x18,0xf8,0xc0,0x00,0x1d,0xfd,0xc0,0x00,0x1f,
  0xff,0xc0,0x00,0x0f,0xff,0xc0,0x00,0x1f,0xff,0xc0,0x00,0x3f,0xff,0xc0,
  0x00,0x3f,0xff,0xc0,0x00,0x3f,0xff,0xc0,0x00,0x3f,0xff,0xc0,0x00,0x3f,
  0xff,0xc0,0x00,0x3f,0xff,0xc4,0x00,0x1f,0xff,0xc9,0x78,0x0f,0xff,0xca,
  0xfc,0x0f,0xff,0x8c,0xff,0xf7,0xff,0x99,0xff,0xf3,0xff,0xe6,0xff,0xf1,
  0xff,0x80,0xff,0xf3,0xff,0x86,0xff,0xf3,0xff,0x89,0xff,0xf7,0xff,0x89,
  0xff,0xf7,0xff,0x92,0xff,0xf7,0xff,0xa4,0xff,0xf3,0xff,0xc4,0x07,0x07,
  0xff,0x88,0x0f,0x89,0xff,0x2a,0x0f,0x80,0xff,0x1c,0x0f,0x80,0xff,0x08,
  0x0f,0x80,0xff,0x00,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xf8,0xff,
  0xff,0xff,0x00,0x00,0x00,0x40,0x06,0x30,0x04,0xd0,0x07,0xf0,0x05,0x58,
  0x05,0x58,0x07,0xd8,0xe6,0xf9,0xff,0xf2,0x95,0xff,0xb5,0x68,0xfd,0x59,
  0xfd,0xff,0x39,0xf8,0x38,0xf0,0x7f,0xf8,0x00,0x00,0x06,0x30,0x04,0xf0,
  0x07,0xf0,0x07,0xf8,0x07,0xf8,0x07,0xf8,0xe7,0xf9,0xff,0xf2,0xff,0xff,
  0xff,0xf8,0xff,0xf9,0xff,0xff,0x3f,0xf8,0x3f,0xf0,0xff,0xff,0xff,0xff,
  0x00,0x00,0x00,0x80,0x00,0x00,0x0f,0xf0,0x00,0xff,0x00,0x00,0x00,0x00,
  0x0f,0x00,0xff,0x0f,0x00,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0x00,0x00,
  0x00,0x00,0x0f,0x0f,0x0f,0x33,0xf0,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x33,
  0xf0,0x00,0x00,0x00,0x03,0x33,0x33,0x33,0xf0,0x00,0xee,0xe0,0x03,0xff,
  0xf3,0x33,0xf0,0x0f,0xee,0xee,0xee,0x33,0x33,0x33,0x00,0xf0,0xec,0xcf,
  0xce,0x03,0x33,0xff,0xff,0xff,0xec,0xff,0xce,0x03,0x3f,0xf3,0xf0,0x00,
  0xef,0xff,0xfe,0x03,0x3f,0xff,0xf0,0x0f,0xee,0xee,0xee,0x0f,0xff,0x33,
  0xff,0xff,0x00,0xaa,0x30,0x0f,0x33,0x3f,0xf0,0x00,0x00,0x33,0x30,0x00,
  0xff,0x3f,0x00,0x00,0xda,0xaa,0xaa,0xaa,0xff,0xaf,0xad,0xdc,0xdd,0xdd,
  0xdd,0xdd,0xdd,0xdd,0xdd,0xdc,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xff,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0xd8,0xd8,
  0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,
  0xd8,0xd8,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xff,0x00,0x00,0x00,0xab,0xab,0xab,0x00,0x00,0xd8,
  0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0x00,0x00,0xff,0xab,0xab,0xab,0xab,
  0xab,0xab,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x00,0x00,0xff,0x00,0xab,0x54,
  0x54,0xfe,0x54,0xab,0x00,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xff,
  0xab,0x54,0xfe,0xfe,0x54,0xab,0x00,0xd8,0xd8,0xff,0xff,0xd8,0xff,0x00,
  0x00,0x00,0xab,0xfe,0xfe,0xfe,0xfe,0xab,0x00,0xd8,0xd8,0xff,0xff,0xff,
  0xff,0x00,0x00,0xff,0xab,0xab,0xab,0xab,0xab,0xab,0x00,0xff,0xff,0xff,
  0xd8,0xd8,0xff,0xff,0xff,0xff,0x00,0x00,0xfd,0xfd,0xd8,0x00,0x00,0xff,
  0xd8,0xd8,0xd8,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xd8,0xd8,0xd8,0x00,
  0x00,0x00,0xff,0xff,0xd8,0xff,0x00,0x00,0x00,0x00,0xf9,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xff,0xff,0xfd,0xff,0xfd,0xf9,0xf9,0xf6,0xf9,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0xf9,0xf6,
  0x00,0x00,0x01,0x00,0x00,0x00,0x09,0xd8,0x00,0x00,0x08,0xd8,0x00,0x00,
  0x00,0x96,0x03,0x71,0xf6,0xc4,0x31,0x94,0x00,0x00,0x00,0x1c,0x00,0x96,
  0x00,0x05,0x69,0x63,0x6c,0x34,0x00,0x00,0x00,0x32,0x69,0x63,0x6c,0x38,
  0x00,0x00,0x00,0x3e,0x49,0x43,0x4e,0x23,0x00,0x00,0x00,0x4a,0x69,0x63,
  0x73,0x23,0x00,0x00,0x00,0x56,0x69,0x63,0x73,0x34,0x00,0x00,0x00,0x62,
  0x69,0x63,0x73,0x38,0x00,0x00,0x00,0x6e,0xbf,0xb9,0xff,0xff,0x00,0x00,
  0x00,0x00,0x03,0x71,0xf4,0x4c,0xbf,0xb9,0xff,0xff,0x00,0x00,0x02,0x04,
  0x03,0x71,0xee,0xa4,0xbf,0xb9,0xff,0xff,0x00,0x00,0x06,0x08,0x03,0x71,
  0xf5,0xa4,0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x0c,0x03,0x71,0xf0,0x78,
  0xbf,0xb9,0xff,0xff,0x00,0x00,0x07,0x50,0x03,0x71,0xf4,0x68,0xbf,0xb9,
  0xff,0xff,0x00,0x00,0x07,0xd4,0x03,0x71,0xf5,0x54
};
#endif  /* HOST_ICON */

/*
 * Finder info for our Icon file,
 * name "Icon^M", type 'rsrc', owner 'RSED', invisible, custom_icon
 *
 */
unsigned char color_cap_icon_fndr[ICON_FNDR_LEN] = { /* FinderInfo */
  0x72,0x73,0x72,0x63,0x52,0x53,0x45,0x44,0x45,0x00,0x00,0x9c,0x00,0x80,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x10,0xda,0x02,0x49,0x63,0x6f,0x6e,
  0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x49,0x63,0x6f,0x6e,0x0d,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xda,0x03,0x33,0xe7,0xfe,0x70,0x33,0xe7,0xfe,
  0xca,0x33,0xe8,0x7d,0xfe,0x00
};

char icon_path[MAXPATHLEN];

/*
 * check if vol/Icon:0d file already in place
 *
 */

int
icon_exists(vol)
char *vol;
{
  sprintf(icon_path, "%s/%s", vol, ICON_NAME);
  if (access(icon_path, R_OK) == 0)
    return(1);

  return(0);
}

/*
 * create three forks of the Icon:0d file
 * in the specified volume root
 *
 */

int
icon_create(vol)
char *vol;
{
  int fd;
  char *cp;
  IDirP pdir;

  /*
   * data fork, zero length
   *
   */
  sprintf(icon_path, "%s/%s", vol, ICON_NAME);
  if ((fd = open(icon_path, O_RDWR|O_CREAT|O_TRUNC, 0644)) < 0)
    return(fd);
  close(fd);

  /*
   * resource fork
   *
   */
  sprintf(icon_path, "%s/.resource/%s", vol, ICON_NAME);
  if ((fd = open(icon_path, O_RDWR|O_CREAT|O_TRUNC, 0644)) < 0)
    return(fd);
  (void)write(fd, color_cap_icon, sizeof(color_cap_icon));
  close(fd);

  /*
   * Finder information
   *
   */
  sprintf(icon_path, "%s/.finderinfo/%s", vol, ICON_NAME);
  if ((fd = open(icon_path, O_RDWR|O_CREAT|O_TRUNC, 0644)) < 0)
    return(fd);
#ifdef USE_MAC_DATES
  (void)write(fd, color_cap_icon_fndr, ICON_FNDR_LEN);
#else  /* USE_MAC_DATES */
  (void)write(fd, color_cap_icon_fndr, ICON_FNDR_WAS);
#endif /* USE_MAC_DATES */
  close(fd);

  /*
   * set the volume comment field
   *
   */
  strncpy(icon_path, vol, sizeof(icon_path));
  if ((cp = (char *)rindex(icon_path, '/')) == NULL)
    return(0);
  *cp++ = '\0';
#ifdef BSD_ICON
  if ((pdir = Idirid(icon_path)) != NILDIR)
    OSSetComment(pdir, cp, INFO_MESSAGE1, strlen(INFO_MESSAGE1));
#else  /* BSD_ICON */
  if ((pdir = Idirid(icon_path)) != NILDIR)
    OSSetComment(pdir, cp, INFO_MESSAGE2, strlen(INFO_MESSAGE2));
#endif /* BSD_ICON */

  return(0);
}
